/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): offroy ________________________.
Contributor(s): ______________________________________.

====================================================================*/
/*
 * Created on 23 mai 2003 by jerome OFFROY (offroy@lifl.fr)
 *
 */

package org.omg.lifl.eclipse.plugin.project.OpenCCM;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.DialogPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.omg.lifl.eclipse.plugin.project.OpenCCM.utils.CVSsrcFSChooser;
import org.omg.lifl.eclipse.plugin.project.OpenCCM.utils.EclipseConfigChooser;
import org.omg.lifl.eclipse.plugin.project.OpenCCM.utils.OpenCCMConfigChooser;
import org.omg.lifl.eclipse.plugin.project.OpenCCM.utils.ProjectMessages;
import org.omg.lifl.eclipse.plugin.project.OpenCCM.utils.ValidatorProject;
import org.omg.lifl.eclipse.plugin.project.utils.AttributeManager;

/**
 * @author LIFL
 *
 */
public class ProjectCreationWizardPage
	extends WizardPage
	implements ValidatorProject {

	private DirectoryDialog fd1;
	private IStatus fCurrStatus;
	private boolean fPageVisible;
	private AttributeManager attributeManager;
	
	private ProjectChooser projectChooser;
	private CVSsrcFSChooser cvsFSchooser;
	private OpenCCMConfigChooser openccmConfigChooser;
    private EclipseConfigChooser eclipseConfigChooser;
	
	/**
	 * @param pageNumber
	 * @param elem
	 */
	public ProjectCreationWizardPage(
		int pageNumber,
		IConfigurationElement elem) {
		super("page" + pageNumber);
		fCurrStatus = createStatus(IStatus.OK, "");
		attributeManager = new AttributeManager(elem);

		setTitle(attributeManager.getAttribute("pagetitle"));
		setDescription(attributeManager.getAttribute("pagedescription"));

		projectChooser = new ProjectChooser(elem);
		cvsFSchooser = new CVSsrcFSChooser(elem);
		openccmConfigChooser = new OpenCCMConfigChooser(elem);
        eclipseConfigChooser = new EclipseConfigChooser(elem);

	}

	/**
	 * @param severity
	 * @param message
	 */
	private static IStatus createStatus(int severity, String message) {
		return new Status(
			severity,
			MainPlugin.getPluginId(),
			severity,
			message,
			null);

	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createControl(Composite parent) {
		Composite composite = new Composite(parent, SWT.NONE);
		RowLayout gd2 = new RowLayout(SWT.VERTICAL);
		composite.setLayout(gd2);
		
		// Group to choose the name of the new project
		projectChooser.viewToSetOpenCCMProjectName(composite,this);
		
		// Group to choose the directory to import in the new project
		cvsFSchooser.viewToSetCVSsrcFS(composite);
		
		// Group to choose the directory of installed ORB
		openccmConfigChooser.viewToSetCCMConfig(composite);
        
        // Group to choose Eclipse WorkBench specific Options
        eclipseConfigChooser.viewToSetEclipseConfig(composite);        
		
		setControl(composite);
	}

	/**
	 * @param string
	 */
	public void validateTextProjectName(String text) {
		IWorkspace workspace = ResourcesPlugin.getWorkspace();
		IStatus status = workspace.validateName(text, IResource.PROJECT);
		if (status.isOK()) {
			if (workspace.getRoot().getProject(text).exists()) {
				status =
					createStatus(
						IStatus.ERROR,
						ProjectMessages.getString(
							"ProjectCreationWizardPage.error.alreadyexists"));
			}
		}
		updateStatus(status);

		projectChooser.set_ProjectName(text);
		
	}

	/**
	 * Updates the status line and the ok button depending on the status
	 * @param status
	 */
	private void updateStatus(IStatus status) {
		fCurrStatus = status;
		setPageComplete(!status.matches(IStatus.ERROR));
		if (fPageVisible) {
			applyToStatusLine(this, status);
		}
	}

	/**
	 * Applies the status to a dialog page
	 * @param page
	 * @param status
	 */
	private static void applyToStatusLine(DialogPage page, IStatus status) {
		String errorMessage = null;
		String warningMessage = null;
		String statusMessage = status.getMessage();
		if (statusMessage.length() > 0) {
			if (status.matches(IStatus.ERROR)) {
				errorMessage = statusMessage;
			} else if (!status.isOK()) {
				warningMessage = statusMessage;
			}
		}
		page.setErrorMessage(errorMessage);
		page.setMessage(warningMessage);
	}

	/*
	 * @see WizardPage#becomesVisible
	 */
	public void setVisible(boolean visible) {
		super.setVisible(visible);
		fPageVisible = visible;
		// policy: wizards are not allowed to come up with an error message
		if (visible && fCurrStatus.matches(IStatus.ERROR)) {
			// keep the error state, but remove the message
			fCurrStatus = createStatus(IStatus.ERROR, "");
		}
		updateStatus(fCurrStatus);
	}

	/**
	 * Returns the name entered by the user
	 */
	public String getName() {
		return projectChooser.get_ProjectName();
	}

	/**
	 * @return
	 */
	public CVSsrcFSChooser getCvsFSchooser() {
		return cvsFSchooser;
	}

	/**
	 * @param chooser
	 */
	public void setCvsFSchooser(CVSsrcFSChooser chooser) {
		cvsFSchooser = chooser;
	}

	/**
	 * @return
	 */
	public OpenCCMConfigChooser getOpenccmConfigChooser() {
		return openccmConfigChooser;
	}

	/**
	 * @param chooser
	 */
	public void setOpenccmConfigChooser(OpenCCMConfigChooser chooser) {
		openccmConfigChooser = chooser;
	}

	/**
	 * @return
	 */
	public AttributeManager getAttributeManager() {
		return attributeManager;
	}

	/**
	 * @param manager
	 */
	public void setAttributeManager(AttributeManager manager) {
		attributeManager = manager;
	}

    /**
     * @return
     */
    public ProjectChooser getProjectChooser() {
        return projectChooser;
    }

    /**
     * @param chooser
     */
    public void setProjectChooser(ProjectChooser chooser) {
        projectChooser = chooser;
    }

    /**
     * @return
     */
    public EclipseConfigChooser getEclipseConfigChooser() {
        return eclipseConfigChooser;
    }

    /**
     * @param chooser
     */
    public void setEclipseConfigChooser(EclipseConfigChooser chooser) {
        eclipseConfigChooser = chooser;
    }

}
