@echo off
rem # ====================================================================
rem #
rem # OpenCCM: The Open CORBA Component Model Platform
rem # Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
rem # Contact: openccm-team@objectweb.org
rem #
rem # This library is free software; you can redistribute it and/or
rem # modify it under the terms of the GNU Lesser General Public
rem # License as published by the Free Software Foundation; either
rem # version 2.1 of the License, or any later version.
rem #
rem # This library is distributed in the hope that it will be useful,
rem # but WITHOUT ANY WARRANTY; without even the implied warranty of
rem # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
rem # Lesser General Public License for more details.
rem #
rem # You should have received a copy of the GNU Lesser General Public
rem # License along with this library; if not, write to the Free Software
rem # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
rem # USA
rem #
rem # Initial developer(s): Philippe Merle.
rem # Contributor(s): Marc Navarro (mnavarro@wins.uva.nl),
rem #                 Christophe Demarey, Jerome Offroy,
rem #                 Briclet Frdric
rem #
rem # ====================================================================
rem #
rem # Starting a demonstration.
rem #
rem # %1: demonstration name.
rem #
rem # ====================================================================

@setlocal

rem # =====================
rem # Compute Command Line.
rem # =====================

set /a nb_args=0
for %%i in (%*) do set /a nb_args=nb_args+1

set demo_name=%1
shift
set /a nb=1
set with_ots="false"
set with_trace="false"
set xml="false"
set run_id=cif
set services_args=
set services_list=
set cad_file=META-INF/%demo_name%.cad

:initial
if "%1" == "-h" goto help
if "%1" == "--ots" goto ots
if "%1" == "--trace" goto trace
if "%1" == "--monolithic" goto mono
if "%1" == "--xml" goto xml
if %nb% == %nb_args% (goto end_command_line) else (goto arguments)

:ots
set with_ots="true"
set services_list=%services_list% Transactional
set services_args=%services_args% --ots
shift
set /a nb=nb+1
goto initial

:trace
set with_trace="true"
set services_list=%services_list% Trace
set services_args=%services_args% --trace
shift
set /a nb=nb+1
goto initial

:mono
set run_id=monolithic
shift
set /a nb=nb+1
goto initial

:xml
set xml="true"
shift
set /a nb=nb+1
goto initial

:arguments
rem set cad_file=META-INF/%1%.cad
shift
set /a nb=nb+1
goto initial

:help
echo usage: start_java ^<demo_name^> [--ots] [--trace] [--monolithic ^| --xml ]
goto :eof

:end_command_line

rem # ==========================
rem # Ant compilation if needed.
rem # ==========================

if not exist generated\idl\%demo_name%.idl (call build)
if not exist archives\%demo_name%.jar (call build)

rem # =============================
rem # Install the OpenCCM Platform.
rem # =============================

call ccm_install

rem # =============================
rem # Start the Name Service.
rem # =============================

call ns_start

rem # ==============================
rem # Start the Transaction Service.
rem # ==============================

if %with_ots% == "true" (call ots_start)

rem # ===========================================
rem # Obtain the IOR of the started Name Service.
rem # ===========================================

call ns_ior > %OpenCCM_CONFIG_DIR%\%demo_name%.tmp
for /f %%i in (%OpenCCM_CONFIG_DIR%\%demo_name%.tmp) do set ior_NS=%%i
del %OpenCCM_CONFIG_DIR%\%demo_name%.tmp

rem # ==================================================
rem # Obtain the IOR of the started Transaction Service.
rem # ==================================================

if %with_ots% == "false" goto :jcs
call ots_ior > %OpenCCM_CONFIG_DIR%\%demo_name%.tmp
for /f %%i in (%OpenCCM_CONFIG_DIR%\%demo_name%.tmp) do set ior_OTS=%%i
del %OpenCCM_CONFIG_DIR%\%demo_name%.tmp

rem # =============================
rem # Start Java Component Servers.
rem # =============================

:jcs
call jcs_start %services_args% ComponentServer1
call jcs_start %services_args% ComponentServer2

rem # =============================
rem # Start Java Node.
rem # =============================
call node_start MainNode

rem # ==================
rem # Start HomeFinder.
rem # ==================
call finder_start

rem # ==================
rem # Run demonstration.
rem # ==================

:run

echo Starting demonstration %demo_name% with [%services_list%] deployment
if %xml% == "false" goto java
call ccm_deploy %services_args% %cad_file%
goto end

:java
set jvm_args=-Dior_ns=%ior_NS%

:ots_service
if %with_ots% == "false" goto trace_service
set jvm_args=%jvm_args% -Dior_ots=%ior_OTS%
set run_id=%run_id%_ots

:trace_service
if %with_trace% == "false" goto deploy
set run_id=%run_id%_trace

:deploy
call launcher exec "%jvm_args%" "--runid %run_id% %demo_name%.xml"

:end
echo The demonstration %demo_name% is ready to be used ...
