/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): Romain Rouvoy, Christophe Demarey.

====================================================================*/

package org.objectweb.ccm.demo1.cif;

/**
 * This is the implementation of the OMG IDL3 demo1::Server component type.
 *
 * This class inherits from the CIF generated skeleton class defined
 * as ServerSessionComposition::ComponentImpl.
 * 
 * The provided for_clients facet is directly implemented by the
 * component class by implementing the demo1::Display interface.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</A>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</A>
 *         <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</A>
 *         <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</A>
 */

public class ServerImpl
     extends org.objectweb.ccm.demo1.ServerSessionComposition.ComponentImpl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The name of the component. */ 
    private String name_;

    /** To refer to the GUI frame. */
    private javax.swing.JFrame frame_;

    /** To refer to the GUI output area. */
    private javax.swing.JTextArea textArea_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /** The default constructor. */
    public
    ServerImpl()
    {
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL Components::EnterpriseComponent
    //
    // ==================================================================

    /**
     * Complete the component configuration.
     *
     * @exception org.omg.Components.InvalidConfiguration
     *            Thrown if the configuration is invalid.
     */
    public void
    configuration_complete ()
    throws org.omg.Components.InvalidConfiguration
    {
        // Checks if the configuration is not completed.
        if(name_ == null)
            throw new org.omg.Components.InvalidConfiguration();

        // Instantiating the GUI.

        // Creates a Swing Frame.
        frame_ = new javax.swing.JFrame(name_ + "'s Server GUI");
        // Sets its size.
        frame_.setSize(400, 300);

        // Creates a text area for displaying inputs.
        textArea_ = new javax.swing.JTextArea(40, 20);
        textArea_.setEditable(false);

        // Constructs and shows the GUI.
        javax.swing.JPanel panel = new javax.swing.JPanel(
                                       new java.awt.BorderLayout()
                                   );
        frame_.getContentPane().add(panel);
        panel.add(new javax.swing.JScrollPane(textArea_),
                  java.awt.BorderLayout.CENTER);
        frame_.pack();
        frame_.show();
    }

    // ==================================================================
    //
    // Methods for the OMG IDL org.omg.Components.SessionComponent
    //
    // ==================================================================

    /**
     * Container callback to signal that the component is removed.
     *
     * @throw org.omg.Components.CCMException For any problems.
     */
    public void
    ccm_remove()
    throws org.omg.Components.CCMException
    {
        // Release the associated frame.
        frame_.dispose();
        frame_ = null;
    }

    // ==================================================================
    //
    // Methods for OMG IDL demo1::CCM_Server_Executor
    //
    // ==================================================================

    /**
     * The mutator method for the attribute name.
     *
     * @param n The name.
     */
    public void
    the_name(String n)
    {
        name_ = n;

        if (frame_ != null)
            frame_.setTitle(name_ + "'s Server GUI");
    }

    /**
     * The accessor method for the attribute name.
     *
     * @return The name.
     */
    public String
    the_name()
    {
        return name_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL demo1::CCM_Display
    //
    // ==================================================================

    /**
     * The print operation of the demo1::Display interface.
     *
     * @param text The message to output.
     */
    public void
    print(String text)
    {
        // Puts the text into the text area.
        textArea_.append(text + "\n");
    }
}
