/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Sylvain Leblanc.
Contributor(s): Christophe Demarey.

====================================================================*/

package DiningPhilosophers.cif;

import DiningPhilosophers.*;

/**
 * This is the implementation of the OMG IDL3 DiningPhilosophers::Observer 
 * component type. This implementation is designed to be used with a PDA.
 *
 * The info event sink is directly implemented
 * by the component class by implementing the
 * DiningPhilosophers::StatusInfoConsumer interface.
 *
 * This class inherits from the skeleton
 * generated by the OpenCCM's CIF to Java mapping generator.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * @author <a href="mailto:Sylvain.Leblanc@lifl.fr">Sylvain Leblanc</a> 
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 2.0
 */

public class ObserverPdaImpl
     extends DiningPhilosophers.ObserverServiceComposition.ObserverService_impl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Stores philosopher's states. */
    private java.util.Hashtable philosopherStates_;

    /** The main GUI. */
    private java.awt.Frame frame_;

    /** The philosopher's panels. */
    private java.util.Hashtable philosopherPanels_;

    /** The panel used to store philosopher's panel GUI. */
    private java.awt.Panel panel_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /** The default constructor. */
    public
    ObserverPdaImpl()
    {
        philosopherPanels_ = new java.util.Hashtable();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL Components::CCMObject interface.
    //
    // ==================================================================

    /**
     * Complete the component configuration and launch the 
     * observer GUI.
     *
     * @exception org.omg.Components.InvalidConfiguration
     *            Thrown if the configuration is invalid.     
     */
    public void
    configuration_complete()
    throws org.omg.Components.InvalidConfiguration
    {
        // Instantiating the GUI.

        // Creates a Swing Frame.
        frame_ = new java.awt.Frame("The Observer's GUI");
        // Sets its size.
        frame_.setSize(400, 300);

        // Constructs and shows the GUI.
        panel_ = new java.awt.Panel(new java.awt.GridLayout(0, 1));
        // Using a scroll pane
        java.awt.ScrollPane scroll_panel = new java.awt.ScrollPane();
        scroll_panel.add(panel_);
        frame_.add(scroll_panel);
        frame_.pack();
        frame_.show();
    }

    // ==================================================================
    //
    // Methods for the OMG IDL org.omg.Components.SessionComponent
    //
    // ==================================================================

    /**
     * Container callback to signal that the component is removed.
     *
     * @throw org.omg.Components.CCMException For any problems.
     */
    public void
    ccm_remove()
    throws org.omg.Components.CCMException
    {
        // Release the associated frame.
        frame_.dispose();
        frame_ = null;
    }

    // ==================================================================
    //
    // Methods for OMG IDL dinner::StatusInfoConsumer
    //
    // ==================================================================

    /**
     * To receive DiningPhilosophers::StatusInfo events.
     *
     * Update the observer GUI with received status informations. 
     */
    public void
    push(StatusInfo event)
    {
        if (! philosopherPanels_.containsKey(event.name)) {
            PhilosopherPdaPanel panel = new PhilosopherPdaPanel(event);
            panel_.add(panel);
            philosopherPanels_.put(event.name, panel);
            frame_.pack();
            frame_.show();
        } else {
            PhilosopherPdaPanel panel;
            panel = (PhilosopherPdaPanel)philosopherPanels_.get(event.name);
            panel.updatePanel(event);
        }
    }
}
