# ====================================================================
#
# OpenCCM: The Open CORBA Component Model Platform
# Copyright (C) 2000-2002 USTL - LIFL - GOAL
# Contact: openccm-team@objectweb.org
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
# USA
#
# Initial developer(s): Areski Flissi.
# Contributor(s): 
#
# ====================================================================
#
#  OMG IDLscript for filetransfer.
#
#  Note: To do before starting this script
#    ir_load filetransfer.idl
#
# ====================================================================

println("Obtaining the Name Service...")

ns = CORBA.ORB.resolve_initial_references("NameService")

# ====================================================================

println("Obtaining the Java Component Servers...")

server1 = ns.resolve([["ComponentServer1",""]])
server2 = ns.resolve([["ComponentServer2",""]])

server1_cs = server1.provide_component_server()
server1_inst = server1.provide_install()

server2_cs = server2.provide_component_server()
server2_inst = server2.provide_install()

# ====================================================================

println("Installing archives...")

server1_inst.install("filetransfer", "./archives/filetransfer.jar")
server2_inst.install("filetransfer", "./archives/filetransfer.jar")

server1_inst.install("openccm_plugins", "./archives/OpenCCM_Plugins.jar")
server2_inst.install("openccm_plugins", "./archives/OpenCCM_Plugins.jar")

# ====================================================================

println("Creating containers...")

server1_cont = server1_cs.create_container([]);
server2_cont = server2_cs.create_container([]);

# ====================================================================

println("Instantiating homes...")

tvh1 = server1_cont.install_home("filetransfer", "org.objectweb.ccm.filetransfer.cif.TVHomeImpl.create_home", [])
tvh2 = server2_cont.install_home("filetransfer", "org.objectweb.ccm.filetransfer.cif.TVHomeImpl.create_home", [])
chanh = server1_cont.install_home("filetransfer", "org.objectweb.ccm.filetransfer.cif.ChannelHomeImpl.create_home", [])
progh = server1_cont.install_home("filetransfer", "org.objectweb.ccm.filetransfer.cif.ProgHomeImpl.create_home", [])


# ====================================================================

println("Instantiating components...")

tv1 = tvh1.create()
tv2 = tvh2.create()
tv3 = tvh1.create()
ch1 = chanh.create()
ch2 = chanh.create()
ch3 = chanh.create()
p1 = progh.create()
p2 = progh.create()
p3 = progh.create()

# ====================================================================

println("Configuring components...")

tv1.the_name = "TV 1"
tv2.the_name = "TV 2"
tv3.the_name = "TV 3"
ch1.the_name = "Channel 1"
ch2.the_name = "Channel 2"
ch3.the_name = "Channel 3"
p1.the_name = "Prog1"
p2.the_name = "Prog2"
p3.the_name = "Prog3"

# ====================================================================

println("Interconnecting components...")

p1.subscribe_to_channels(ch1.get_consumer_from_prog())
ch1.subscribe_to_tvs(tv1.get_consumer_from_channels())
p2.subscribe_to_channels(ch2.get_consumer_from_prog())
ch2.subscribe_to_tvs(tv2.get_consumer_from_channels())
p3.subscribe_to_channels(ch3.get_consumer_from_prog())
ch3.subscribe_to_tvs(tv3.get_consumer_from_channels())

# ====================================================================

println("Configuration completion...")

tv1.configuration_complete()
tv2.configuration_complete()
tv3.configuration_complete()
ch1.configuration_complete()
ch2.configuration_complete()
ch3.configuration_complete()
p1.configuration_complete()
p2.configuration_complete()
p3.configuration_complete()

# ====================================================================
