/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): ___________________________________________________.

====================================================================*/


package org.objectweb.ccm.Deployment;

import org.objectweb.ccm.CORBA.*;
/**
 * Interface definition : ServerActivator
 * 
 * @author OpenORB Compiler
 */
public class ServerActivatorImpl extends org.omg.Components.Deployment.ServerActivatorPOA 
{
    /*Last attributed component server number */		
    private int last_cs_num;
    
    /*Table for storage of cs*/
    private java.util.Hashtable c_servers;
    
    /*Table for storage of each component server*/
    private java.util.Hashtable process;
    
    /*Name of the local computer*/
    private String localName ;
    
    /*OpenCCM directory */
    private String openCCMHomeDir ;
    
    /*Temporary boolean definition to resolve PDA distinction*/
    private boolean PDA;

    /**
     * Main constructor of ServerActivator two parameter are necessary
     * to be able to start new component server.
     * @param localName The currnet computer name which use to fix component
     * server name.
     * @param openCCMHomeDir Which use to refer component server XML launcher descriptor
     */ 
    public ServerActivatorImpl(String localName, String openCCMHomeDir){
	last_cs_num=0;
	c_servers= new java.util.Hashtable();
	process=new java.util.Hashtable();
	this.localName=localName;
	this.openCCMHomeDir=openCCMHomeDir;
		this.PDA=false;
    }
    
    /**
     * Peciliar constructor for PDA use
     */
    public ServerActivatorImpl(String localName, String openCCMHomeDir,boolean PDA){
        last_cs_num=0;
        c_servers= new java.util.Hashtable();
        process=new java.util.Hashtable();
        this.localName=localName;
        this.openCCMHomeDir=openCCMHomeDir;
        this.PDA=PDA;
    }   
 
    /**
     * Operation create_component_server :
     * Three main stape buildare lead in this method :
     *		- Build arguments for the command to execute
     *		- Build a new launcher (with this argument) whitch start the Component Server
     * 		- Retrieve information ComponentServer reference and process
     * TODO: Consider the config value
     */
    public org.omg.Components.Deployment.ComponentServer 
	create_component_server(org.omg.Components.ConfigValue[] config)
        throws org.omg.Components.CreateFailure, org.omg.Components.Deployment.InvalidConfiguration
    {
	
        org.omg.Components.Deployment.ComponentServer serverRef =null;
        try{
            /* args [0] nom de la machine locale
             * args[1] non du fichier a ecrire
             */ 
            
            //Build the name of new Component Server base on a number 
            String server_name="ComponentServer"+last_cs_num++;
            
	    if(!PDA){
		//Path to JCS xml launcher descriptor
		String targetToLaunch =openCCMHomeDir+"/xml/launcher/JCS.xml" ;
        	//Arguments need to launch a Component Server
		String args []= {server_name,localName,"-ORBInitRef NameService="+TheORB.getORB()
				 .object_to_string(TheORB.resolve_initial_reference("NameService"))};
        	//Launching the new Component server throw launcher tool
		org.objectweb.util.launcher.Launcher launcher= 
		    new org.objectweb.util.launcher.Launcher(targetToLaunch,"jcs_interne",args);
		
		java.io.BufferedReader br = new java.io.BufferedReader(
							    new java.io.InputStreamReader(
							    launcher.getProcess().getInputStream()));
		
		// Skip the first line which signal component server ready
		br.readLine();
		
		// retrieve the component server reference throw the outputed String ior
		//System.out.println("CS IOR:"+launcher.getServerIOR());
		serverRef=org.omg.Components.Deployment.
		    ComponentServerHelper.narrow(TheORB.getORB().string_to_object(br.readLine()));	
		
		//store the reference of component server process 
		process.put(server_name+localName,launcher.getProcess());
		//all is OK 
		System.out.println("Component Server ["+server_name+localName+"] started"); 
	    }          
	    else{
		//PDA version avoiding to create new JVM
		org.objectweb.ccm.Components.ConfigValueImpl[] config1=
		    new org.objectweb.ccm.Components.ConfigValueImpl[1];
		
		org.omg.CORBA.Any any= org.objectweb.ccm.CORBA.TheORB.getORB().create_any();
		
                    //store the component server name in config parameter
		any.insert_string(server_name+localName); 
		config1[0]= new org.objectweb.ccm.Components.ConfigValueImpl("ServerName",any);
		//creation of new Component Server into the current JVM memory space
		ComponentServerImpl csi = new ComponentServerImpl(config1);  
		// Activate it.
		org.omg.CORBA.Object ref =
		    csi._this_object(org.objectweb.ccm.CORBA.TheORB.getORB());
		//Retrieve the corba reference                            
		serverRef=org.omg.Components.Deployment.ComponentServerHelper.narrow(ref);
		// Obtain the Name Service.
		org.objectweb.ccm.CORBA.NamingContext ns =
		    org.objectweb.ccm.CORBA.TheNameService.getNamingContext();
                
		// Bind the server object in the name service.
		ns.rebind(server_name+localName, serverRef);
	    }
            
            //store the reference of the component server
            c_servers.put(server_name+localName,serverRef);
            //return component server ref        
            return  serverRef;
	}
	catch(Exception e){e.printStackTrace();}
	
	// return serverRef null if this statement is reached
	return serverRef;
    }
    
    /**
     * Operation remove_component_server, by stoping him and killing his process.
     * The component server give in parameter must be created by this Server Activator 
     * else a RemoveFailure exception will be raise.
     * Call the remove() function on the server pass to parameter.
     */
    public void remove_component_server(org.omg.Components.Deployment.ComponentServer server)
        throws org.omg.Components.RemoveFailure{
        
      	//Retrive configuration informations
        org.omg.Components.ConfigValue[] config =server.configuration();	
	// the first config value of component server must be component server name			
        if(!config[0].name.equals("ServerName"))
	    throw new org.omg.Components.RemoveFailure();	
	
	 // test if this Component server was start by this Server Activator	
        if(!c_servers.containsKey(config[0].value.extract_string()))
	    throw new org.omg.Components.RemoveFailure();	
	
	try{
	    //Start the removing processing for the component server
	    server.remove();      	
	    if(!PDA){
        	//remove the reference of the process
		Process p=(Process)process.remove(config[0].value.extract_string());
		//Remove the reference of the component server 
	       	c_servers.remove(config[0].value.extract_string());
		//Kill the process
		p.destroy();
	    }
	}
	catch(Exception e)
	    {
		e.printStackTrace();
		System.out.println(e.getMessage());
	    }
    }
    
    
    /**
     * Kill the component server process and destroy the delete the CS reference 
     * from the list. Used to break the remove circle reference.
     */
    public void remove_component_server_interne(org.omg.Components.Deployment.ComponentServer server)
	throws org.omg.Components.RemoveFailure
    {
	//Retrive configuration informations
        org.omg.Components.ConfigValue[] config =server.configuration();	
	// the first config value of component server must be component server name			
        if(!config[0].name.equals("ServerName"))
	    throw new org.omg.Components.RemoveFailure();	
	// test if this Component server was start by this Server Activator	
        if(!c_servers.containsKey(config[0].value.extract_string()))
	    throw new org.omg.Components.RemoveFailure();	
	
	try{
	    //Remove the reference of the component server 
            c_servers.remove(config[0].value.extract_string());
            
            if(!PDA){
		// Retrieve the process reference               
    	       	Process p=(Process)process.remove(config[0].value.extract_string());
    	       	//Kill the process
    	       	p.destroy();
	    }
	}
	catch(Exception e){e.printStackTrace();
	System.out.println(e.getMessage());
	}
	
    }
    
    /**
     * Operation get_component_servers: Return a set of all active components servers
     * created by this Server Activator
     */
    public org.omg.Components.Deployment.ComponentServer[] get_component_servers(){
    	return (org.omg.Components.Deployment.ComponentServer[])
	    c_servers.values().toArray(new org.omg.Components.Deployment.
		ComponentServer[c_servers.size()]);
    }
    
}
