/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA & USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

import org.omg.CORBA.InterfaceDef;

/**
 * Implementation of the CORBA::InterfaceDef reference.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.4
 */

public class InterfaceDef_ref
     extends IDLType_ref
{
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * TODO.
     */
    public
    InterfaceDef_ref(IRObject_impl owner)
    {
        // Calls the IDLType_ref constructor.
        super(owner);
    }

    /**
     * TODO.
     */
    public
    InterfaceDef_ref(IRObject_impl owner,
                     InterfaceDef_impl object)
    {
        // Calls the IDLType_ref constructor.
        super(owner, object);
    }

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Static public methods.
    //
    // ==================================================================

    /**
     * Cuts the dependency to a set of interfaces.
     */
    static public void
    cutDependencies(InterfaceDef_ref[] itfs)
    {
        if (itfs != null)
        {
            for(int i=0; i<itfs.length; i++)
                itfs[i].cutDependency();
        }
    }

    /**
     * Computes the CORBA::RepositoryIdSeq for an InterfaceDef_ref[].
     */
    static public String[]
    computeRepositoryIdSeq(InterfaceDef_ref[] itfs)
    {
        String[] result = new String[itfs.length];

        for(int i=0; i<itfs.length; i++)
            result[i] = itfs[i].getId();

        return result;
    }

    /**
     * Computes the CORBA::InterfaceDefSeq for an InterfaceDef_ref[].
     */
    static public InterfaceDef[]
    computeInterfaceDefSeq(InterfaceDef_ref[] itfs)
    {
        InterfaceDef[] result = new InterfaceDef[itfs.length];

        for(int i=0; i<itfs.length; i++)
             result[i] = itfs[i].asInterfaceDef();

        return result;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Obtains its implementation.
     */
    public InterfaceDef_impl
    getImpl()
    {
        return (InterfaceDef_impl)(super.getBaseImpl());
    }

    /**
     * Obtains its CORBA::InterfaceDef reference.
     */
    public InterfaceDef
    asInterfaceDef()
    {
        if (getTarget() == null) return null;
        return getImpl().asInterfaceDef();
    }

    /**
     * Obtains its repository ID.
     */
    public String
    getId()
    {
        if (getTarget() == null) return "";
        return getImpl().id();
    }
}
