/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): ___________________________________________________.

====================================================================*/

package org.objectweb.ccm.deploytool;
/**
 * TearDownTool allow to tear down an assembly from a console or
 * from java code.
 *
 * @author <a href="mailto:frederic.briclet@lifl.fr">Briclet Frederic</a>
 *
 * @version 0.1
 */
public class TearDownTool {
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * Default constructor
     * @param assemblyToTearDown the assembly name
     */
    public TearDownTool(String assemblyToTearDown){
        try{
            tearDown(assemblyToTearDown);
        }    
        catch(Exception e)
            {
                System.err.println("Cannot tear_down the assembly"+assemblyToTearDown);
            }
        
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================
    /**
     * Method allowing to tear down an assembly 
     * @param assemblyToTearDown the assembly name
     */
    public void 
    tearDown(String assemblyToTearDown)
    throws Exception
    {
        org.omg.CosNaming.NameComponent[] ncomp
                        = new org.omg.CosNaming.NameComponent[1];
        
        ncomp[0]= new org.omg.CosNaming.NameComponent(assemblyToTearDown,"");
        //Resolve the assembly name with the name service 
        org.omg.CORBA.Object obj = org.objectweb.ccm.CORBA.TheNameService
                                            .getNamingContext()
                                            .getNamingContext()
                                            .resolve(ncomp);
        //Retrieves the assembly
        org.omg.Components.Deployment.Assembly assemblyObject =
                    org.omg.Components.Deployment.AssemblyHelper.narrow(obj);
        //TearDown the assembly
        assemblyObject.tear_down();
        
    }

    public static void main(String args[]){
        //Initialize the ORB
        org.objectweb.ccm.CORBA.TheORB.initialize(args);
        //create a new TearDownTool to realise the tear_down process
        new TearDownTool(args[0]);    
        
        org.objectweb.ccm.CORBA.TheORB.destroy();
    }
    
}
