/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): Christophe Contreras.

====================================================================*/

package org.objectweb.ccm.descriptor.componentassembly.ccm;

import org.objectweb.ccm.descriptor.componentassembly.beans.*;
import org.objectweb.ccm.descriptor.componentassembly.*;

public class ComponentassemblyDeployer{


    /**
     *    Do the first part of the deployment process. Call recursively the deployPartial
     *  method on the componentfiles and partitionning children. Recursives call are
     *    done throw the extension manager tool. Calling the methods directly on the
     *    ComponentDeployer and partioningDeployer link to the children. The same technic
     *    are use on the other recursives calls.
     *  This first must deploy all the required instance.
     */
    public void deployPartial(java.util.Hashtable deployTable,Componentassembly cai){
    // Load the imbricated CSD and CAD, using the extension manager tool
    if(cai.getComponentfiles()!=null)
        ((ComponentfilesDeployer)
         ((ComponentfilesBean)cai.getComponentfiles())
         .getExtensionManager().getExtensionByName("ccm.Deployer")
         .getInstance()).deployPartial(deployTable,cai.getComponentfiles());


    // Home deployment and instantiation of components
    if(cai.getPartitioning()!=null)
        ((PartitioningDeployer)
         ((PartitioningBean)cai.getPartitioning())
         .getExtensionManager().getExtensionByName("ccm.Deployer")
         .getInstance()).deploy(deployTable,cai.getPartitioning());
    }

    /**
     *    Do the second part of the deployment, this function establishes the connection
     * between  components.A first  recursiv call of deployFinal function on connections
     * are done to establish the primary connection. The second call od deployfinal on
     *    ComponentFiles establish the imbricated CAD connections. In the same way the
     * extension manager toool is used for recursives calls.
     */
    public void deployFinal(java.util.Hashtable deployTable,Componentassembly cai){
    // Establish imbricated connections
    if(cai.getComponentfiles()!=null)
        ((ComponentfilesDeployer)
         ((ComponentfilesBean)cai.getComponentfiles())
         .getExtensionManager().getExtensionByName("ccm.Deployer")
         .getInstance()).deployFinal(deployTable,cai.getComponentfiles());
    //Establish primary connections
    if(cai.getConnections()!=null)
        ((ConnectionsDeployer)
         ((ConnectionsBean)cai.getConnections())
         .getExtensionManager().getExtensionByName("ccm.Deployer")
         .getInstance()).deploy(deployTable,cai.getConnections());

    }

    /**
     * First tear down method to break connections
     */
    public void tearDownConnections(java.util.Hashtable deployTable, ComponentassemblyImpl cai){
    //Break connections
    if(cai.getConnections()!=null)
        ((ConnectionsDeployer)
         ((ConnectionsBean)cai.getConnections())
         .getExtensionManager().getExtensionByName("ccm.Deployer")
         .getInstance()).tearDown(deployTable,cai.getConnections());

    //Remove Components informations
    if(cai.getComponentfiles()!=null)
        ((ComponentfilesDeployer)
         ((ComponentfilesBean)cai.getComponentfiles())
         .getExtensionManager().getExtensionByName("ccm.Deployer")
         .getInstance()).tearDownConnections(deployTable,cai.getComponentfiles());
    }

    /**
     *Final tear down method in charge to remove all the installed component
     */
    public void tearDownInstantiations(java.util.Hashtable deployTable, ComponentassemblyImpl cai){

    //Delete instance
    if(cai.getPartitioning()!=null)
        ((PartitioningDeployer)
         ((PartitioningBean)cai.getPartitioning())
         .getExtensionManager().getExtensionByName("ccm.Deployer")
         .getInstance()).tearDown(deployTable,cai.getPartitioning());
    //Remove Components informations
    if(cai.getComponentfiles()!=null)
        ((ComponentfilesDeployer)
         ((ComponentfilesBean)cai.getComponentfiles())
         .getExtensionManager().getExtensionByName("ccm.Deployer")
         .getInstance()).tearDownInstantiations(deployTable,cai.getComponentfiles());
    }
}
