/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): ___________________________________________________.

====================================================================*/

package org.objectweb.ccm.descriptor.componentassembly.ccm;

import org.objectweb.ccm.descriptor.componentassembly.beans.*;
import org.objectweb.ccm.descriptor.componentassembly.*;
import org.objectweb.ccm.descriptor.properties.*;
import org.objectweb.ccm.descriptor.properties.beans.*;
import org.omg.CORBA.* ;

public class ComponentinstantiationDeployer{

    /**
     *
     * The deploy method goals are multiples. It must instantiate the component,
     * setup their property with the cpf file linked and finally register the component
     * in the define services.
     * Finally each instance are store into the deployTable with the instance id for key.
     * It must be point out that none more recursiv call are done. Componentinstantiation
     * is a deployer three execution leaf.
     */    
    public void deploy(java.util.Hashtable deployTable,
               org.omg.Components.CCMHome homeDeployed,
               Componentinstantiation cii)
    {
    try{
        //instantiate the component
        org.omg.Components.CCMObject new_component =
        org.omg.Components.KeylessCCMHomeHelper
        .narrow(homeDeployed)
        .create_component();

        try {// Test if a property file is define
        if(cii.getComponentproperties()!=null)
            setupComponentProperty(deployTable,new_component,cii);
        }
        catch(Exception e){
        e.printStackTrace();
                       /*((org.objectweb.util.cmdline.api.Console)deployTable.get("__Console__"))
                  .message("Aucun fichier de config existant pour cet instance "+cii.getId());*/
        }

        // storing the reference in the DeployInfo extension of the descriptor
        ( (ComponentinstantiationDeployInfo)
          ((ComponentinstantiationBean)cii).getExtensionManager()
          .getExtensionByName("ccm.DeployInfo")
          .getInstance()).setCorbaReference(new_component);

        //Register component in registration service
        registerComponent(new_component,cii);

        // Store the instance to deployTable for future use
        deployTable.put(cii.getId(),new_component);
        }
        catch(Exception e){ e.printStackTrace();}

    }

    /**
     * Iternal method to register component to each service defined.
     * Simply browse the component registration list and apply the name
     * service registration.
     * TODO : Another registration
     */
    private void registerComponent(org.omg.Components.CCMObject new_component ,
                   Componentinstantiation cii )throws Exception {
    // browse all register component
    org.omg.CORBA.Object to_register=(org.omg.CORBA.Object)new_component;
        for (java.util.Iterator comp_reg_i= cii.getRegistercomponentList().iterator() ;
         comp_reg_i.hasNext() ; )
        {
        RegistercomponentBean comp_registration=
            (RegistercomponentBean) comp_reg_i.next();

        // test if a part of component must be register
        if(comp_registration.getProvidesidentifier()!=null){
            to_register=new_component.provide_facet(comp_registration
                                .getProvidesidentifier().getValue());
        }else if(comp_registration.getConsumesidentifier()!=null){
            to_register=new_component.get_consumer(comp_registration
                               .getConsumesidentifier().getValue());
        }

        //Browse all the NameService registration tag
        for (java.util.Iterator ns_reg_i =
             comp_registration.getRegisterwithnamingList().iterator() ;
             ns_reg_i.hasNext() ; )
            //Proceed with binding
            org.objectweb.ccm.CORBA.TheNameService
            .getNamingContext().rebind(((Registerwithnaming)ns_reg_i.next()).getName()
                           , (org.omg.CORBA.Object) to_register);
        }
    }

    /**
     * Internal method to setup the component properties.
     * The first part process the simple properties, the second part process
     * the sequence and structure property.
     * TODO: the sequence and structure setup process
     */
    private void setupComponentProperty(java.util.Hashtable deployTable,
                    org.omg.Components.CCMObject new_component ,
                    Componentinstantiation cii)throws Exception{

    // open the property file for the instance if defined
        java.io.File propertiesFile= new java.io.File ((String)deployTable.get("BaseDir")+
                               cii.getComponentproperties()
                               .getFileinarchive().getName());

        // retrieving the properties object descriptor
        PropertiesBean propertiesObject =PropertiesBeanImpl.unmarshalBean(propertiesFile);
    //setup the simple properties
    setupSimpleProperty(propertiesObject,new_component);
    //setup the sequence properties
    /*setupSequenceListProperty(new_component,propertiesObject)*/
    }


    /**
     * Internal method to setup the simple component properties. This method simply
     * browse simple properties list and setup its
     */
    private void setupSimpleProperty(PropertiesBean propertiesObject,
                     org.omg.Components.CCMObject new_component)throws Exception{
    //browse all the component simple properties
    for (java.util.Iterator i = propertiesObject.getSimpleList().iterator() ; i.hasNext() ; )
        {
        SimpleBean property= (SimpleBean) i.next();

        // build a set request
            org.omg.CORBA.Request request
            = new_component._request("_set_"+ property.getName());
            //Process the simple property
            processSimple(property,request.add_in_arg()) ;
            // invocation of the set request
            request.invoke();
        }
    }

    /**
     * Internal method to setup the sequence property. This methods browse the sequence
     * list property and recursivly setup the imbricated properties
     */
    /*private void setupSequenceListProperty()throws Exception {
      for (java.util.Iterator i = propertiesObject.getSequenceList().iterator();i.hasNext() ; ){
      org.objectweb.ccm.descriptor.properties.SequenceBean property
      = (org.objectweb.ccm.descriptor.properties.SequenceBean) i.next();

      org.omg.CORBA.Request request
      = new_component._request("_set_"+ property.getName());

      request.add_in_arg().insert_any(processSequence(property));
      request.invoke();
      }
      }*/

    /**
     * Retrieve the TCKind object relative to the typeCodeID
     */
    private TCKind getTypeCode(String typeCodeId) {

    //System.out.println("Type rechercher :"+typeCodeId);
    if(typeCodeId.equals("boolean"))
        return TCKind.from_int(org.omg.CORBA.TCKind._tk_boolean);
    else if(typeCodeId.equals("char"))
        return TCKind.from_int(org.omg.CORBA.TCKind._tk_char);
    else if(typeCodeId.equals("double"))
        return TCKind.from_int(org.omg.CORBA.TCKind._tk_double);
    else if(typeCodeId.equals("float"))
        return TCKind.from_int(org.omg.CORBA.TCKind._tk_float);
    else if(typeCodeId.equals("short"))
        return TCKind.from_int(org.omg.CORBA.TCKind._tk_short);
    else if(typeCodeId.equals("long"))
        return TCKind.from_int(org.omg.CORBA.TCKind._tk_long);
    else if(typeCodeId.equals("objref"))
        return TCKind.from_int(org.omg.CORBA.TCKind._tk_objref);
    else if(typeCodeId.equals("octet"))
        return TCKind.from_int(org.omg.CORBA.TCKind._tk_octet);
    else if(typeCodeId.equals("string"))
        return TCKind.from_int(org.omg.CORBA.TCKind._tk_string);
    else if(typeCodeId.equals("ulong"))
        return TCKind.from_int(org.omg.CORBA.TCKind._tk_ulong);
    else if(typeCodeId.equals("ushort"))
        return TCKind.from_int(org.omg.CORBA.TCKind._tk_ushort);
    else if(typeCodeId.equals("sequence"))
        return TCKind.from_int(org.omg.CORBA.TCKind._tk_sequence);
    else if(typeCodeId.equals("struct"))
        return TCKind.from_int(org.omg.CORBA.TCKind._tk_struct);

    return null ;    // UNKNOWN type
    }


    /*private Any processSequence(SequenceBean si){
      // Process simple parameter
      org.omg.CORBA.DynSequence sequence=
      org.objectweb.ccm.CORBA.TheORB.getORB()
      .create_dyn_sequence(org.objectweb.ccm.CORBA.TheORB.getORB().
      create_sequence_tc(0,org.objectweb.ccm.CORBA.TheORB.getORB().
      get_primitive_tc(getTypeCode(si.getType()))));

      for (java.util.Iterator i = si.getSimpleList().iterator() ; i.hasNext() ; )
      {
      org.objectweb.ccm.descriptor.properties.SimpleBean property
      = (org.objectweb.ccm.descriptor.properties.SimpleBean) i.next();

      sequence.insert_any(processSimple(property)) ;
      }
      //process sequence parameter
      for (java.util.Iterator i = si.getSequenceList().iterator();i.hasNext() ; ){
      org.objectweb.ccm.descriptor.properties.SequenceBean property
      = (org.objectweb.ccm.descriptor.properties.SequenceBean) i.next();

      any.insert_any(processSequence(property,new Any()));
      }

      return sequence.to_any();
      }*/


    /*private void processStruct(SimpleBean property ){
      }*/

    private Any processSimple(SimpleBean property , Any any){
    //Any any=org.objectweb.ccm.CORBA.TheORB.getORB().create_any();
    if (property.getType().equals( "double"))
        any.insert_longlong(Long.parseLong(property.getValue().getValue()));
    //else if(property.getType().equals( "objref"))
    //     any.insert_Object(property.getValue().getValue());
    else if(property.getType().equals( "boolean"))
        any.insert_boolean(Boolean.getBoolean(property.getValue().getValue()));
    else if(property.getType().equals( "char"))
        any.insert_char(property.getValue().getValue().charAt(0));
    else if(property.getType().equals( "float"))
        any.insert_float(Integer.parseInt(property.getValue().getValue()));
    else if(property.getType().equals( "short"))
        any.insert_short(Short.parseShort(property.getValue().getValue()));
    else if(property.getType().equals( "long"))
        any.insert_long(Integer.parseInt(property.getValue().getValue()));
        else if(property.getType().equals( "octet"))
        any.insert_octet(Byte.parseByte(property.getValue().getValue()));           
        else if(property.getType().equals( "string"))
        any.insert_string(property.getValue().getValue());
        else if(property.getType().equals( "ulong"))
        any.insert_ulong(Integer.parseInt(property.getValue().getValue()));
        else if(property.getType().equals( "ushort"))
        any.insert_ushort(Short.parseShort(property.getValue().getValue()));
        
        return any;
    }
    

    
    /**
     *Remove from deployTable the component reference , unbind the component
     */
    public void tearDown(java.util.Hashtable deployTable, Componentinstantiation cii)throws Exception{
    
    for (java.util.Iterator cri= cii.getRegistercomponentList().iterator();cri.hasNext() ; )
        {
        RegistercomponentBean cr=(RegistercomponentBean) cri.next();
        
        //Browse all the NameService registration tag
        for (java.util.Iterator ns_reg_i = cr.getRegisterwithnamingList().iterator() ; 
             ns_reg_i.hasNext() ; )
            //Proceed with unbinding
                org.objectweb.ccm.CORBA.TheNameService    
                    .getNamingContext().unbind(
                                   ((Registerwithnaming)ns_reg_i.next())
                                   .getName());
        }
    deployTable.remove(cii.getId());
    (( (ComponentinstantiationDeployInfo)
       ((ComponentinstantiationBean)cii).getExtensionManager()
       .getExtensionByName("ccm.DeployInfo")
       .getInstance()).getCorbaReference()).remove();
    
    }
          
}
