/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): ___________________________________________________.

====================================================================*/

package org.objectweb.ccm.descriptor.componentassembly.ccm;

import org.objectweb.ccm.descriptor.componentassembly.beans.*;
import org.objectweb.ccm.descriptor.componentassembly.*;

public class ConnecteventDeployer {
    
    /* Interanal exception used to manage the connection*/
    private class CannotResolveReferenceException extends Exception{};
    
    /*Cookie whicth store the connection information*/
    private org.omg.Components.Cookie cookie; 
    
    /* Reference to subscriber */ 
    private org.omg.Components.CCMObject subscriber;
    
    /**
     * Proceed the connectevent connection :
     *            - Retrieve the consumedObject
     *            - Retrieve the subscriber
     *            - Connect the consumer or subscribe the consumer
     * ConnectEventDeployer is a leaf of execution deployer three no more
     * recursives call to deploy method are done.
     * Both emitsport and publishesport are supported but following the
     * specification just the first will be process if both are defined.
     */
    public void deploy(java.util.Hashtable deployTable,Connectevent cei)
    {
    try{
        
            // Get the consumes object
            org.omg.Components.EventConsumerBase consumedObject=null ;
            if(cei.getConsumesport()!=null)
        {
            consumedObject=
            proceedConsumesPort(deployTable , cei.getConsumesport());
        }
            else if(cei.getExistinginterface()!=null)
        {
            consumedObject=
            proceedExistingInterface(cei.getExistinginterface());
        }                                    
        
        if(cei.getPublishesport()!=null){
        //retrieve the subscriber
        subscriber=proceedPublishesPort(deployTable,cei.getPublishesport());
        //Subscribe to the object
        cookie=subscriber.subscribe(cei.getPublishesport()
                        .getPublishesidentifier()
                        .getValue()
                        ,consumedObject);
        }
        else if(cei.getEmitsport()!=null){
        //retrieve the subscriber
        subscriber=proceedEmitsPort(deployTable,cei.getEmitsport());
        //Subscribe to the object
        subscriber.connect_consumer(cei.getEmitsport()
                        .getEmitsidentifier()
                        .getValue()
                        ,consumedObject);
        }
        
    }
    catch(Exception e){ 
        System.out.println("Cannot establishes connection");
        e.printStackTrace();
    }
    }
    
    /**
     * Resolve and existing interface and return it like the consumed object.
     * If any interface can be found an exception is thrown
     */
    private 
    org.omg.Components.EventConsumerBase 
    proceedExistingInterface(Existinginterface ei)throws Exception
    {
    return org.omg.Components.EventConsumerBaseHelper
        .narrow(resolveReference(ei.getFindby()));        
    }
    
    /**
     * Internal method to resolve and retrieve the publisher. First it try to find 
     * the publisher reference with the componentinstantiationref, if it is null
     * it try to resolve the reference with registration services.
     * If the both way failed an exception is thrown.
     */
    private org.omg.Components.CCMObject proceedPublishesPort
    (java.util.Hashtable deployTable, Publishesport pp)throws Exception
    {
    // retrieve the publisher throws the instantiationref if it's null
    if(pp.getComponentinstantiationref()!=null)
        return (org.omg.Components.CCMObject)
        deployTable.get(pp.getComponentinstantiationref().getIdref());
    // try to retrieve the reference throw registration services    
    return org.omg.Components.CCMObjectHelper.narrow(resolveReference(pp.getFindby())) ;        
    }    
    
    /**
     * Internal method to resolve and retrieve the emitter. First it try to find 
     * the emitter reference with the componentinstantiationref, if it is null
     * it try to resolve the reference with registration services.
     * If the both way failed an exception is thrown.
     */
    private org.omg.Components.CCMObject proceedEmitsPort
    (java.util.Hashtable deployTable, Emitsport ep)throws Exception
    {
    
    if(ep.getComponentinstantiationref()!=null)
            return (org.omg.Components.CCMObject)
                deployTable.get(ep.getComponentinstantiationref().getIdref());
    
    return org.omg.Components.CCMObjectHelper.narrow(resolveReference(ep.getFindby())) ;        
    }    
    
    /**
     * Internal method to resolve and retrieve the comsumedObject.
     * First it try to find the consumer reference with the componentinstantiationref,
     * if it is null, it try to resolve the reference with registration services.
     * If the both way failed an exception is thrown.
     * If the consumer is found the method return the consumed object invoking 
     * on the consumer the get_consumer method with consumeidentifier to parameter. 
     */
    private  org.omg.Components.EventConsumerBase proceedConsumesPort
    (java.util.Hashtable deployTable , Consumesport cp)throws Exception
    {
    
    org.omg.Components.CCMObject consumer ;
    
    if(cp.getComponentinstantiationref()!=null)
        consumer =(org.omg.Components.CCMObject)
        deployTable.get(cp.getComponentinstantiationref().getIdref());                
    
    else consumer=org.omg.Components.CCMObjectHelper.narrow(resolveReference(cp.getFindby()));
    
    return consumer.get_consumer(cp.getConsumesidentifier().getValue());
    }
    
    /**
     * Internal method to resolve a reference point out by a findby component.
     * First it with try to find the object with a stringifield reference.
     * Secondly it try to resolve it with name service.
     * Note: If two ways to find are defined only the stringifiedobjectref will
     * be proceed, according to the specification only on ways would be defined.
     * TODO: processing with Trader
     */   
    private org.omg.CORBA.Object resolveReference(Findby fb)throws Exception
    {
    //TODO : reslving component with trader and stringifield reference
    if(fb!=null ) {
        if(fb.getStringifiedobjectref()!=null)
        return org.objectweb.ccm.CORBA.TheORBSingleton.getORB()
            .string_to_object(fb.getStringifiedobjectref().getValue());
        else if(fb.getNamingservice()!=null)
        return proceedWithNameService(fb.getNamingservice());
    }
    throw new CannotResolveReferenceException();
    }
    
      
    /**
     * Internal fonction to resolve and retrieve a reference with the name service. 
     * The reference look up is defined in the Nameservice object tag. If any object 
     * can be found with this reference an exception is thrown.
     */   
    private org.omg.CORBA.Object proceedWithNameService( Namingservice service)
    throws Exception
    {
    
    org.omg.CosNaming.NameComponent[] ncomp
        = new org.omg.CosNaming.NameComponent[1];
    
    ncomp[0]= new org.omg.CosNaming.NameComponent(service.getName(),"");
    
    org.omg.CORBA.Object obj = org.objectweb.ccm.CORBA.TheNameService
        .getNamingContext()
        .getNamingContext()
        .resolve(ncomp);
    return obj;
    //return org.omg.Components.CCMObjectHelper.narrow(obj);
    }
    
    
    
    /**
     *Tear down connection : unsubscrive the publisher
     *TODO: Rewrite the tear_down
     */
    public void tearDown(java.util.Hashtable deployTable,Connectevent cei)
    {
    try{
        // call the unsubscribe method
        if(cei.getPublishesport()!=null){
        subscriber.unsubscribe(cei.getPublishesport()
                       .getPublishesidentifier()
                       .getValue()
                       ,cookie);
        }else if(cei.getEmitsport()!=null){
        subscriber.disconnect_consumer(cei.getEmitsport()
                           .getEmitsidentifier().getValue());
        }
        
    }
    catch(Exception e){e.printStackTrace(); }
    }
}
