/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): ___________________________________________________.

====================================================================*/

package org.objectweb.ccm.descriptor.componentassembly.ccm;

import org.objectweb.ccm.descriptor.componentassembly.beans.*;
import org.objectweb.ccm.descriptor.componentassembly.*;
import org.objectweb.ccm.Deployment.components.HFinder.*;

public class ConnectinterfaceDeployer {
    
    //Internal exception to warn of impossibility to found a HomeInterface
    private class CannotResolveHomeInterfaceException extends Exception {};
    //Internal exception to warn of impossibility to resovle a reference
    private class CannotResolveReferenceException extends Exception{};
    
    
    /*Cookie whitch store the connection information*/
    private org.omg.Components.Cookie cookie; 
    
    /* Reference of the user*/
    org.omg.Components.CCMObject user;     
     
    /**
     * Establish the connectinterface connection:
     *        -Retrieve the consumedObject
     *        -Retrieve the user user
     *        -Connect the user with the provided object
     *    All Providesport , Homeinterface, Componentsupportedinterface and 
     *    existing interface are supported, but only one can be defined only the first
     *    will be proceed.
     *    If one of communicating part can't be found the connection can be establish
     * the process is aborted, informing of the problem.    
     */
    public void deploy(java.util.Hashtable deployTable,Connectinterface cii)
    {
         try{
            System.out.println("Etablishing connection");
            org.omg.CORBA.Object providedObject=null ;
        /*Proceed a Providesport tag*/
        if(cii.getProvidesport()!=null){
        providedObject = proceedProvidesport(deployTable,cii.getProvidesport());
        }/*Proceed the homeinterface tag*/
        else if(cii.getHomeinterface()!=null){
        providedObject =proceedHomeInterface(deployTable,cii.getHomeinterface());    
        }/*Proceed the component supported interface tag*/
        else if(cii.getComponentsupportedinterface()!=null){
        providedObject= proceedComponentSupportedInterface(deployTable,
                                   cii.getComponentsupportedinterface());
        }/*proceed the existing interface tag*/
        else if(cii.getExistinginterface()!=null){
        providedObject= proceedExistingInterface(cii.getExistinginterface());
        }    
        
        if(providedObject==null)
        throw new Exception("Cannot establishes connection");
        
        /*get the uses object*/
        user=proceedUsesport(deployTable,cii.getUsesport());
        
            /*org.omg.Components.ReceptacleDescription [] desc=user.get_all_receptacles();
          for(int i=0; i<desc.length;i++)
          System.out.println("Recepacle:"+i+"  "+desc[i].name);
          // call the connect method
          System.out.println("Nom du port :"+cii.getUsesport().getUsesidentifier().getValue());*/
            cookie=user.connect(cii.getUsesport().getUsesidentifier().getValue(),providedObject);    
    }
    catch(Exception e)
        {
        System.out.println("Cannot establishes connection");
        e.printStackTrace();
        }
    }
    
    
    /**
     * Resolve and existing interface and return it like the consumed object.
     * If any interface can be found an exception is thrown
     */
    private org.omg.CORBA.Object proceedExistingInterface(Existinginterface ei)throws Exception
    {
    return resolveReference(ei.getFindby())    ;    
    }
    
    
    /**
     * Internal method to resolve and retrieve the supportedinterface. 
     * First it try to find supportedinterface reference with 
     * the componentinstantiationref, if it is null
     * it try to resolve the reference with registration services.
     * If the both way failed an exception is thrown.
     */
    private org.omg.Components.CCMObject proceedComponentSupportedInterface
    (java.util.Hashtable deployTable,Componentsupportedinterface csi)throws Exception
    {
    if(csi.getComponentinstantiationref()!=null)
        return (org.omg.Components.CCMObject)
        deployTable.get(csi.getComponentinstantiationref().getIdref());
    return (org.omg.Components.CCMObject)resolveReference(csi.getFindby());        
    }
    
    /**
     *Internal method to resolve and retrieve the user.
     * First it try to find usesport reference with 
     * the componentinstantiationref, if it is null
     * it try to resolve the reference with registration services.
     * If the both way failed an exception is thrown.
     */
    private org.omg.Components.CCMObject proceedUsesport
    (java.util.Hashtable deployTable,Usesport upi)throws Exception
    {
    if(upi.getComponentinstantiationref()!=null)
        return (org.omg.Components.CCMObject)
        deployTable.get(upi.getComponentinstantiationref().getIdref());
    return org.omg.Components.CCMObjectHelper.narrow(resolveReference(upi.getFindby()))    ;
    }
    
    /**
     * Internal method to resolve and retrieve the providedobject.
     * First it try to resolve the provider reference with componentinstantiationref.
     * If it is null then it try to resolve the provider reference with registration services.
     * If the both way failed an exception is thrown.
     * If the provider is found the function return the providedobject invoking the
     * provided_facet on the provider with providesidentifier to parameter
     */
    private org.omg.CORBA.Object proceedProvidesport
    (java.util.Hashtable deployTable , Providesport ppi)throws Exception
    {
    
    if(ppi.getComponentinstantiationref()!=null)
        return ((org.omg.Components.CCMObject)
            (org.omg.CORBA.Object )deployTable.get(ppi.getComponentinstantiationref().getIdref()))
        .provide_facet(ppi.getProvidesidentifier().getValue());
    return org.omg.Components.CCMObjectHelper.narrow(resolveReference(ppi.getFindby()))
        .provide_facet(ppi.getProvidesidentifier().getValue());    
    }
    
    /**
     * Internal method to resolve and retrieve the Homeinterface.
     * First it try to resolve the reference with componentinstantiationref.
     * If it is null then it try to resolve the reference with registration services.
     * If the both way failed it try to found the interface with the homefinder service.
     * If all attempt failled an CannotResolveHomeInterfaceException is raised
     * If an HomeInterface is found it's return.
     */
    private org.omg.Components.CCMHome proceedHomeInterface
    (java.util.Hashtable deployTable , Homeinterface hii)throws Exception
    {
    /*try to resolve with homeplacementref*/
    if(hii.getHomeplacementref()!=null)
        return (org.omg.Components.CCMHome)
        deployTable.get(hii.getHomeplacementref().getIdref());
    
    if(hii.getFindby()!=null)
        try{//try to resolve reference with registration services
        return org.omg.Components.CCMHomeHelper.narrow(resolveReference(hii.getFindby()));
        }
        catch(Exception e){
        // search the reference with the homefinder
        return proceedWithHomeFinder(hii.getFindby().getHomefinder());
        }
    // cannot found the interface raise an exception
    throw new CannotResolveHomeInterfaceException();
    }
    
    /**
     * Internal method to resolve a reference point out by a findby component.
     * First it with try to find the object with a stringifield reference.
     * Secondly it try to resolve with name service.
     * Note: If two ways to find are defined only the stringifiedobjectref will
     * be proceed, following the specification only on ways would be defined.
     * TODO: processing with Trader
     */   
    private org.omg.CORBA.Object resolveReference(Findby fb)throws Exception{
    //TODO : reslving component with trader and stringifield reference
    if(fb!=null ) {
        if(fb.getNamingservice()!=null)
        return proceedWithNameService(fb.getNamingservice());
        else if(fb.getStringifiedobjectref()!=null)
        return org.objectweb.ccm.CORBA.TheORBSingleton.getORB().
            string_to_object(fb.getStringifiedobjectref().getValue());
    }
    throw new CannotResolveReferenceException();
    }
    
    
    
    /**
     * Internal method in charge to found the reference throw the homefinder.
     * It search first the HomeFinder reference into the nameService and secondary
     * use this homefinder reference to try to resolve the home name precised.
     * If the home interface is found it is return, else an exception is raised
     */
    private  org.omg.Components.CCMHome proceedWithHomeFinder(Homefinder hf)throws Exception
    {
    String homeName=hf.getName();
    
    /*Retrieve the homefinder service*/
    org.omg.CosNaming.NameComponent[] ncomp
                = new org.omg.CosNaming.NameComponent[1];
    
    ncomp[0]= new org.omg.CosNaming.NameComponent("HomeFinder","");
    
    org.omg.CORBA.Object obj = org.objectweb.ccm.CORBA.TheNameService
        .getNamingContext()
        .getNamingContext()
        .resolve(ncomp);    
    
    HFinderManager hm=HFinderManagerHelper.narrow(obj);
    HomeFinder hFinder= hm.provide_the_home_finder();
    /*Invoke a search by name into the homefinder*/
    return hFinder.find_home_by_name(homeName);
    //return null;
    }
      
    
     /**
      * Internal fonction to resolve and retrieve a reference with the name service. 
      * The reference look up is defined in the Nameservice object tag. If any object 
      * can be found with this reference an exception is thrown.
      */   
    private org.omg.CORBA.Object proceedWithNameService( Namingservice service)
    throws Exception
    {
    org.omg.CosNaming.NameComponent[] ncomp
        = new org.omg.CosNaming.NameComponent[1];
    
    ncomp[0]= new org.omg.CosNaming.NameComponent(service.getName(),"");
    
    org.omg.CORBA.Object obj = org.objectweb.ccm.CORBA.TheNameService
        .getNamingContext()
        .getNamingContext()
        .resolve(ncomp);
    return obj;
    //return org.omg.Components.CCMObjectHelper.narrow(obj);
    }
    
     
    /**
     *Disconect the user 
     */ 
    public void tearDown(java.util.Hashtable deployTable,Connectinterface cii)
    {
       try{
        user.disconnect(cii.getUsesport().getUsesidentifier().getValue(),cookie);
    }
    catch(Exception e){e.printStackTrace();}
    }  
    
}
