/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): ___________________________________________________.

====================================================================*/

package org.objectweb.ccm.descriptor.componentassembly.ccm;

//Importation of the needed packages
import org.objectweb.ccm.descriptor.componentassembly.beans.*;
import org.objectweb.ccm.descriptor.componentassembly.*;
import org.objectweb.ccm.descriptor.softpkg.*;
import org.objectweb.ccm.descriptor.softpkg.beans.*;

/**
 * Deployer class in charge to resolve the specified destination. 
 * It supports to destination kind: CS and Node. <br>
 * <ul>
 *  <li>CS destination refer to the older association betweeen  
 *  a component server and a component installation. This server is 
 *  started bu JCS_start script.The destination precised is in fact the name 
 *  CS started on target host.
 *  </li>
 * 
 * <li>Node destination refer to the ServerActivator/ComponentInstallation
 * association defined in the CCM specification. The destination precised 
 * is in fact the name of the started node on target host.
 * <li>
 * </ul>
 * @author <a href="mailto:frederic.briclet@lifl.fr">Briclet Frederic</a>
 *
 * @version 0.1
 */

public class DestinationDeployer {
    
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    
    /** A server which hold a ComponentServer and a ComponentInstantiation*/
    private org.objectweb.ccm.Deployment.Server server ;
    /** The target server activator */
    private org.omg.Components.Deployment.ServerActivator sa;
    /** The target component installation*/
    private org.omg.Components.Deployment.ComponentInstallation ci;
    /** Namecomponent use to found object*/
    private org.omg.CosNaming.NameComponent [] ncomp;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================
    public DestinationDeployer(){
        server=null;
        sa=null;
        ci=null;
        ncomp=new org.omg.CosNaming.NameComponent [1];
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================
    
    /**
     * TODO : Process exception mecanism
     * Definition of method allowing to retrieve a component server
     * reference pointed by the destination.<br>
     *  <destination>CS:"ServerName"</destination> <br>
     *  <destination>Node:"NodeName"</destination> <br>
     *
     * @param target the target destination
     * @return the target component server
     **/
    public 
    org.omg.Components.Deployment.ComponentServer
    getComponentServer(java.util.Hashtable deployTable,Destination target)
    {
        try{
            String destination=target.getValue();
            
            if(destination.trim().startsWith("CS:"))
                return getServer(destination).provide_component_server();
            
            //initalize the component server configuration                          
            org.omg.Components.ConfigValue[] config
                = new org.omg.Components.ConfigValue[0];
            
            if (destination.trim().startsWith("NODE:")){
                
                // create and return the component server reference                      
                return getServerActivator(destination)
                    .create_component_server(config);
            }
            if(destination.trim().startsWith("DCI")){
                return getServerActivatorDCI(deployTable,destination)
                    .create_component_server(config);
            }
        }
        catch(Exception e){e.printStackTrace();}        
        return null;
    }
    
    /**
     * Definition method provide the component server pointed
     * by the destination.<br>
     *  <destination>CS:"ServerName"</destination> <br>
     *  <destination>Node:"NodeName"</destination> <br>
     *
     * @param target the target destination
     * @return the target component installation
     */
     public 
     org.omg.Components.Deployment.ComponentInstallation
     getComponentInstallation(java.util.Hashtable deployTable,Destination target)
    {
        String destination=target.getValue();
        try{   
            //Destination refer server composition:CS+CI
            if(destination.trim().startsWith("CS:"))
                //return Component Installation segmnet  
                return getServer(destination).provide_install();
        
            //Destination refer to a node 
            if (destination.trim().startsWith("NODE:"))
                return getComponentInstallation(destination);                    
            if (destination.trim().startsWith("DCI:"))
                return getComponentInstallationDCI(deployTable,destination);
        }
        catch(Exception e)
            {
                System.out.println("Cannot retrieve the ComponentInstallation:"+
                                   e.getMessage());
                e.printStackTrace();
            }
        return null;         
    }
    


    private org.omg.Components.Deployment.ComponentInstallation
    getComponentInstallationDCI(java.util.Hashtable deployTable,String destination){
        String dest=destination.substring(destination.indexOf("DCI:")+4,  
                                              destination.length());
        return null; //deployTable.get("DCI");
    }
    
    private org.omg.Components.Deployment.ServerActivator
    getServerActivatorDCI(java.util.Hashtable deployTable,String destination){
        return null;
    }
    

    /**
     * Remove a component server.
     *
     * @param cs the componentserver to remove
     */
    public void 
    removeComponentServer(org.omg.Components.Deployment.ComponentServer cs)
    throws Exception
    {
        if(sa!=null)
        sa.remove_component_server(cs);
    }
    
    
    // ==================================================================
    //
    // Private methods.
    //
    // ==================================================================

    /**
     * Private method allowing to retrieve server reference with the name 
     * service.
     *
     * @param destination the target server name to retrive
     * @return the server found
     */
    private 
    org.objectweb.ccm.Deployment.Server 
    getServer(String destination)
    throws Exception
    {
        if(server==null){
            String dest=destination.substring(destination.indexOf("CS:")+3,  
                                              destination.length());
            dest=dest.trim();
            //retrive server composition
            server=org.objectweb.ccm.Deployment.ServerHelper.narrow(getRef(dest)); 
        }
        return server;
    }
    
    /**
     *Internal method to retrieve ServerActivator.
     *
     * @param destination the target server activator name to retrive
     * @return the server activator found
     */
    private 
    org.omg.Components.Deployment.ServerActivator
    getServerActivator(String destination)
    throws Exception
    {
        if(sa==null){
            String dest=destination.substring(destination.indexOf("NODE:")+5,  
                                              destination.length());
            dest=dest.trim();
            //Retrieve the Server Activator Reference
            sa=org.omg.Components.Deployment.ServerActivatorHelper.
                narrow(getRef("ServerActivator"+dest)); 
        }
        return sa;                               
    }
    
    /**
     *Internal method to retrieve ComponentInstallation
     *
     * @param destination the target component installation  name to retrive
     * @return the component installation found     
     */
    private org.omg.Components.Deployment.ComponentInstallation
    getComponentInstallation(String destination)
    throws Exception
    {
        if(ci==null){
            String dest=destination.substring(destination.indexOf("NODE:")+5,  
                                              destination.length());
            dest=dest.trim();
            //Retrieve the Server Activator Reference
            ci=org.omg.Components.Deployment.ComponentInstallationHelper.
        narrow(getRef("ComponentInstallation"+dest)); 
        }
        return ci;                               
    }
    
    /**
     * Provide a corba reference according to the name give in 
     * parameter.
     *
     * @param ref the name of corba reference to retrieve
     * @return the ref corba found
     **/
    private  org.omg.CORBA.Object 
    getRef(String ref)
    throws Exception
    {
        ncomp[0]= new org.omg.CosNaming.NameComponent(ref,"");
        return  org.objectweb.ccm.CORBA.TheNameService
        .getNamingContext()
        .getNamingContext()
        .resolve(ncomp);  
    }
}
