/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): ___________________________________________________.

====================================================================*/

package org.objectweb.ccm.descriptor.componentassembly.ccm;

//import the needed packages
import org.objectweb.ccm.descriptor.componentassembly.beans.*;
import org.objectweb.ccm.descriptor.componentassembly.*;
import org.objectweb.ccm.descriptor.softpkg.*;
import org.objectweb.ccm.descriptor.softpkg.beans.*;
import org.objectweb.ccm.Deployment.components.HFinder.*;
/**
 * HomeplacementDeployer install a home and configure it. Il also browse
 * all the componentinstantiation to build.
 *
 * @author <a href="mailto:frederic.briclet@lifl.fr">Briclet Frederic</a>
 *
 * @version 0.1
 */
public class HomeplacementDeployer {
    
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    
    /**Internal exception for deployment managing*/        
    private class CannotSetupHomeException extends Exception{};
    /**The component server target */ 
    private org.omg.Components.Deployment.ComponentServer server_cs ;
    /**The component installation target*/
    private org.omg.Components.Deployment.ComponentInstallation server_inst;
    /**The container target */
    private org.omg.Components.Deployment.Container server_cont;
    /**The home produced*/
    private org.omg.Components.CCMHome homePlaced;
    /**The HomeFinder cookie registration*/
    java.util.Stack cookies;
    
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================
    
    /**
     * Deploy is in charge to instance and setup the home on the precise 
     * destination. Special destiantion parameter is given for homeplacement
     * imbricated into hostcollocation or processcollocation.
     * There are 8 main traitments into this method :<br>
     *     <ul>
     *       <li>Retrieve the component_instantiation</li>
     *       <li>Gather information on the home instance( with csd file)</li>
     *       <li>Installation of needed package on target destination</li>
     *       <li>create a container for this home with correct configurations</li>
     *       <li>Instantiate the home </li>
     *       <li>Store the home reference into deployTable, extension manager tool</li>
     *       <li>Register the home on nameservice and homefinder if necessary </li>
     *       <li>Instantiate the component contained by this home</li>
     *     </ul>  
     * @param deployTable the internal table use to store information.
     * @param dest the target destination
     * @param hi Homeplacement information to use.
     */
    
    public void 
    deploy(java.util.Hashtable deployTable, 
           Destination dest,
           Homeplacement hi )
    {
        try {
            //if dest parameter is null the homeplacement 
            //destination must be consider
            if(dest==null)
                dest=hi.getDestination();
            
            //setup the componentinstantiation server
            setComponentInstantiation(deployTable,dest);

            setupSoftpkgDependency(deployTable,hi); 

            // setup the container and install homes with container configuration.
            homePlaced =setupHome(deployTable,hi) ;
            
            // storing the reference in the DeployInfo extension of the descriptor
            ( (HomeplacementDeployInfo)
              ((HomeplacementBean) hi).getExtensionManager().
              getExtensionByName("ccm.DeployInfo")
              .getInstance() ).setCorbaReference(homePlaced);

            // store the reference in the partition
            deployTable.put(hi.getId(),homePlaced);
            
            //Register the home in registration services definded                         
            proceedHomeRegistration(homePlaced,hi);         
            
            /*setup the componentinstantiation*/
            proceedComponentInstantiation(deployTable,homePlaced,hi)        ;
            System.out.println("Home "+hi.getId()+"is  deployed");  
        }
        catch(Exception e){
            System.err.println("Probleme occured during the deployement of"+hi.getId());
            e.printStackTrace();
        }
    }
    
    /**
     *Tear down the homeplacement but dont remove the file archive
     *
     * @param deployTable the internal table use to store information.
     * @param dest the target destination
     * @param hi Homeplacement information to use.
     */
    public void 
    tearDown(java.util.Hashtable deployTable, 
             Destination dest ,
             Homeplacement hi)
    {
        try{
            if(dest==null)
                dest=hi.getDestination();
            // Remove all the instantiation        
            for (java.util.Iterator i= hi.getComponentinstantiationList().iterator() ;
                 i.hasNext() ; ){
                //((ComponentinstantiationImpl)i.next()).tearDown(deployTable);
                ComponentinstantiationBean current=(ComponentinstantiationBean)i.next();
                try{
                    ((ComponentinstantiationDeployer)
                     current
                     .getExtensionManager().getExtensionByName("ccm.Deployer")
                     .getInstance()).tearDown(deployTable,current);
                }
                catch(Exception e){System.out.println("Cannot remove the instance");
                e.printStackTrace();
                }
            }                                    
            
            String ho="";
            try
                {
                    
                    if(hi.getRegisterwithnamingList()!=null&&
                       hi.getRegisterwithnamingList().size()!=0){
                        
                        for(java.util.Iterator i=hi.getRegisterwithnamingList().iterator();
                            i.hasNext();){
                            ho=((RegisterwithnamingImpl)i.next()).getName();
                            org.objectweb.ccm.CORBA.TheNameService.getNamingContext()
                                .unbind(ho);
                        }
                    }
                }
            catch(Exception e){
                System.err.println("Cannot unbind "+ho+" from name service");                    
            }
            
            try{
                org.omg.CosNaming.NameComponent ncomp[]=new org.omg.CosNaming.NameComponent[1];
                    ncomp[0]= new org.omg.CosNaming.NameComponent("HomeFinder","");
                
                org.omg.CORBA.Object obj = org.objectweb.ccm.CORBA.TheNameService
                    .getNamingContext()
                    .getNamingContext()
                    .resolve(ncomp);    
                
                HFinderManager hf=HFinderManagerHelper.narrow(obj);
                HomeRegistration hr= hf.provide_the_home_recorder();
                // proceed home finder registrations
                while(cookies!=null&&!cookies.isEmpty())
                    hr.unregister_home((org.omg.Components.Cookie)cookies.pop());               
            }
            catch(Exception e)
            {
                System.out.println("Cannot found home hinder,the home hasn't be unreferenced");
            }
            
            // Remove the component server
            ((DestinationDeployer)((DestinationBean)dest)
             .getExtensionManager().getExtensionByName("ccm.Deployer")
             .getInstance()).removeComponentServer(server_cs);
            
            deployTable.remove(hi.getId());
        }
        catch(Exception e)
            {
                System.out.println(e.getMessage());
                e.printStackTrace();
            }
    } 
    


        
    // ==================================================================
    //
    // private methods.
    //
    // ==================================================================
    
    /**
     * Setup the dependencies defined in softpkg tags
     */
    private void
    setupSoftpkgDependency(java.util.Hashtable deployTable,Homeplacement hi)
    throws java.io.IOException,
           org.omg.Components.Deployment.InvalidLocation,
           org.omg.Components.Deployment.InstallationFailure
    {
        
        java.io.File file;
        String absolutePath;
        
        SoftpkgBean softpkg=(SoftpkgBean)deployTable.get(hi.getComponentfileref().
                                                         getIdref());
        
        if(softpkg.getDependencyList()==null)
            return;
        // parse the softpkg dependency list and install its
        for (java.util.Iterator dep_itrator = softpkg.getDependencyList().iterator() ;
             dep_itrator.hasNext() ; )
            {
                org.objectweb.ccm.descriptor.softpkg.Dependency dep
                    = (org.objectweb.ccm.descriptor.softpkg.Dependency) 
                      dep_itrator.next();
                
                if (dep.getAction().equals("install")
                    && (dep.getFileinarchive()!=null)
                    && (dep.getFileinarchive().getName()!=null)
                    && (dep.getName()!=null)
                    && (dep.getName().getValue()!=null)
                    )
                    {  
                        file=new java.io.File(dep.getFileinarchive().getName());
                        if(file.exists())
                            absolutePath=file.getCanonicalPath();
                        else absolutePath=dep.getFileinarchive().getName();
                        server_inst.install(dep.getName().getValue(),
                                            absolutePath);                         
                    }
            }
        
    }

    
    
    /**
     * Interanal method in charge to init retrieve the componentServe needed
     * for homeplacement deployment.
     *
     * @param dest the component installation location
     */
    private void 
    setComponentInstantiation(java.util.Hashtable deployTable,Destination dest)
    throws Exception
    {
        // Resolving nameservice problem  Obtain the component servers.    
        server_cs=((DestinationDeployer)((DestinationBean)dest)
           .getExtensionManager().getExtensionByName("ccm.Deployer")
           .getInstance()).getComponentServer(deployTable,dest);
        server_inst=((DestinationDeployer)((DestinationBean)dest)
             .getExtensionManager().getExtensionByName("ccm.Deployer")
             .getInstance()).getComponentInstallation(deployTable,dest);             
        
    }

    /**
     * Internal method in charge to setup the container  and to create the home.
     * 
     * @param deployTable the internal table use to store information.
     * @param hi Homeplacement to setup.  
     */
    private org.omg.Components.CCMHome 
    setupHome(java.util.Hashtable deployTable,Homeplacement hi) 
    throws Exception 
    {
        
        ImplementationBean implemToInstall=
            (ImplementationBean)deployTable.get(hi.getComponentimplref().getIdref());
        
        java.io.File file;
        String absolutePath;

        // Install archives necessary to this homeplacement only the first archive will be install
        for( java.util.Iterator i=implemToInstall.getCodeList().iterator();i.hasNext();){
            CodeBean cb=(CodeBean)i.next();
            
            
            file=new java.io.File(cb.getFileinarchive().getName());
            
            if(file.exists())
                absolutePath=file.getCanonicalPath();
            else absolutePath=cb.getFileinarchive().getName();
            System.out.println("Archive to  downloaded :"+absolutePath);
            server_inst.install(implemToInstall.getId(),absolutePath);					     
        }
        // parse the implementation dependency list and install its
        if(implemToInstall.getDependencyList()!=null)
            {
                for (java.util.Iterator dep_itrator = implemToInstall.getDependencyList().iterator() ;
                     dep_itrator.hasNext() ; )
                    {
                        org.objectweb.ccm.descriptor.softpkg.Dependency dep
                            = (org.objectweb.ccm.descriptor.softpkg.Dependency) dep_itrator.next();
                        
                        if (dep.getAction().equals("install")
                            && (dep.getFileinarchive()!=null)
                        && (dep.getFileinarchive().getName()!=null)
                            && (dep.getName()!=null)
                            && (dep.getName().getValue()!=null)
                            )
                            {  
                                file=new java.io.File(dep.getFileinarchive().getName());
                                if(file.exists())
                                absolutePath=file.getCanonicalPath();
                                else absolutePath=dep.getFileinarchive().getName();
                                server_inst.install(dep.getName().getValue(),
                                                absolutePath);                         
                            }
                    }            
                
            }
    
        org.omg.Components.ConfigValue[] config
            = new org.omg.Components.ConfigValue[0];
        
   
        //create the container and save its configuration
        server_cont= server_cs.create_container(config);	

        // create the home with container configuration
        try
            {        
                for( java.util.Iterator i=implemToInstall.getCodeList().iterator();i.hasNext();){
                    CodeBean  code =(CodeBean) i.next();
                    if(code.getEntrypoint()!=null)
                        return server_cont.install_home(implemToInstall.getId(),
                                                        code.getEntrypoint().getValue(), config);
                } 
                
               
            }catch(Error err){System.out.println(err.getMessage());}
	
        throw new CannotSetupHomeException();    	
  }


    /**
     * Internal method to setup the componentinstantiation children of this homeplacement.
     * The deploy method is call on each browsed elements
     *
     * @param deployTable the internal table use to store information.
     * @param homePlaced the home to use for instantiation
     * @param hi ComponentInstantiation information.  
     */
    private void 
    proceedComponentInstantiation(java.util.Hashtable deployTable ,
                                  org.omg.Components.CCMHome homePlaced,
                                  Homeplacement hi) 
    throws Exception 
    {
        // browse the componentinstantiation taf
        for (java.util.Iterator i= hi.getComponentinstantiationList().iterator() ;
             i.hasNext() ; )
            {
                ComponentinstantiationBean current=(ComponentinstantiationBean)i.next();
                ((ComponentinstantiationDeployer)
                 current
                 .getExtensionManager().getExtensionByName("ccm.Deployer")
                 .getInstance()).deploy(deployTable,homePlaced,current);
            }        
    }
    
    /**
     * Internal to proceed the registration to registration service like nameservice
     * and homefinder. 
     * TODO : Support a trader
     * TODO : Simplify the test of homefinder list
     *
     * @param homePlaced the home to register
     * @param hi the register information
     */
    private void 
    proceedHomeRegistration(org.omg.Components.CCMHome homePlaced,
                            Homeplacement hi )
    throws Exception
    {
    // proceed with binding
    if(hi.getRegisterwithnamingList()!=null&&
       hi.getRegisterwithnamingList().size()!=0){
        
        for(java.util.Iterator i=hi.getRegisterwithnamingList().iterator();i.hasNext();){
        org.objectweb.ccm.CORBA.TheNameService
            .getNamingContext()
            .rebind( ((RegisterwithnamingImpl)i.next()).getName()
                 , (org.omg.CORBA.Object) homePlaced);
        }
    }
    
    // test if some homefinder registration must be done
    if(hi.getRegisterwithhomefinderList()!=null&&
       hi.getRegisterwithhomefinderList().size()>0){
        
        org.omg.CosNaming.NameComponent[] ncomp
        = new org.omg.CosNaming.NameComponent[1];
        
        ncomp[0]= new org.omg.CosNaming.NameComponent("HomeFinder","");
        
        org.omg.CORBA.Object obj = org.objectweb.ccm.CORBA.TheNameService
        .getNamingContext()
        .getNamingContext()
        .resolve(ncomp);    
        
        HFinderManager hf=HFinderManagerHelper.narrow(obj);
        HomeRegistration hr= hf.provide_the_home_recorder();
        
        cookies=new java.util.Stack();
        
        // proceed home finder registrations
        for(java.util.Iterator i=hi.getRegisterwithhomefinderList().iterator();
            i.hasNext();)
            {
                String hnf=((RegisterwithhomefinderImpl)i.next()).getName();
                cookies.push(hr.register_home(homePlaced,"","",hnf));
            }
    }
    }   
    
}
