/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): .

====================================================================*/

package org.objectweb.corba.generator.cif.lib;

/** To access all the AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;

/** To access all the AST Scope. */
import org.objectweb.corba.ast.api.Scope;

/** To access all the AST Composition. */
import org.objectweb.corba.ast.api.CompositionDecl;

/** To access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/** To use Vector. */
import java.util.Vector;

/** To use GenerationException. */
import org.objectweb.corba.generator.common.lib.GenerationException;


/**
 * This class generates CIF interfaces in the Interface Repository.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</A>
 */

public class CIF_IDLGenerator
     extends org.objectweb.corba.generator.common.lib.GeneratorBase
  implements org.objectweb.corba.generator.cif.api.CIF_IDLGenerator
{

    // ===========================================================
    //
    // Internal methods.
    //
    // ===========================================================

    /** Declarations creates for CIF. **/
    private Vector cif_decls_;

    /** CIF Declarations to remove from IR. **/
    private Vector decls_to_remove_;

    // ===========================================================
    //
    // Constructors.
    //
    // ===========================================================

    /**
     * The default constructor.
     *
     * @param ast - The Abstract Syntax Tree.
     **/

    public CIF_IDLGenerator(org.objectweb.corba.ast.api.AST ast)
    {
        // Call the GeneratorBase constructor
        super(ast);

        cif_decls_ = null;
        decls_to_remove_ = null;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Generate CIF interfaces from AST.
     * IR must be initialized first with method "initRepository()".
     *
     * @param scope - The scope to map.
     *
     * @return The list of mapped modules.
     **/
    public Vector
    declare_cif(Scope scope)
    throws GenerationException
    {
        Vector vect = null;
        Declaration[] decls = null;
        org.objectweb.corba.ast.api.NativeDecl nativ = null;

        cif_decls_ = new Vector();
        decls_to_remove_ = new Vector();

        /** Get all psdl abstract storage home and map to native **/
        vect = getAllDeclarations(scope, DeclarationKind.dk_abstract_storage_home);
        decls = (Declaration[])vect.toArray(new Declaration[0]);
        for (int i=0; i<decls.length; i++)
        {
            // System.out.println("decl : "+decls[i].getName());
            nativ = ((org.objectweb.corba.ast.api.AbstractStorageHomeDecl)decls[i]).getMapping();
            cif_decls_.add(nativ);
            decls_to_remove_.add(nativ);
        }

        /** Get all psdl abstract storage type and map to native **/
        vect = getAllDeclarations(scope, DeclarationKind.dk_abstract_storage_type);
        decls = (Declaration[])vect.toArray(new Declaration[0]);
        for (int i=0; i<decls.length; i++)
        {
            nativ = ((org.objectweb.corba.ast.api.AbstractStorageTypeDecl)decls[i]).getMapping();
            cif_decls_.add(nativ);
            decls_to_remove_.add(nativ);
        }

        /** Get all compositions contained in the file scope **/
        vect = getAllDeclarations(scope, DeclarationKind.dk_composition);
        decls = (Declaration[])vect.toArray(new Declaration[0]);

        // For each composition, call declare_cif()
        for (int i=0; i<decls.length; i++)
        {
            CompositionDecl comp = ((CompositionDecl)decls[i]);
            cif_decls_.add( comp.declareCIF() );
            decls_to_remove_.add(comp);
        }

        // return new created declarations
        return cif_decls_;
    }


    /**
     * Remove CIF declarations from I.R.
     **/
    public void
    remove_decls()
    {
        Declaration decl = null;
        Vector vect = new Vector();

        // First remove Compositions
        for (int i=0; i<decls_to_remove_.size(); i++)
        {
            decl = (Declaration)decls_to_remove_.get(i);
            try
            {
                ((org.objectweb.corba.ast.api.CompositionDecl)decl).destroy();
            }catch(ClassCastException e){
                vect.add(decl);
            }
        }

        // remove remaining declarations (i.e. Compositions)
        for (int i=0; i<vect.size(); i++)
        {
            decl = (Declaration)vect.get(i);
            decl.destroy();
        }
    }
}
