/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ________________________________________.

====================================================================*/

package org.objectweb.corba.generator.psdl.lib;

/** To access AST. */
import org.objectweb.corba.ast.api.StorageHomeFactoryDecl;
import org.objectweb.corba.ast.api.AbstractStorageTypeDecl;
import org.objectweb.corba.ast.api.StorageTypeDecl;
import org.objectweb.corba.ast.api.StorageTypeStateMemberDecl;

/** To access Java AST. */
import org.objectweb.corba.generator.java.ast.api.*;
import org.objectweb.corba.generator.java.ast.lib.*;

/** Others. */
import org.objectweb.corba.generator.common.lib.GenerationException;


/**
 * This class generates the mapping for a PSDL factory.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</A>
 */
public class PSDLFactoryMapping
  implements org.objectweb.corba.generator.psdl.api.PSDLFactoryMapping
{

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The PSDL factory Declaration. */
    private StorageHomeFactoryDecl factory_;

    // ===========================================================
    //
    // Constructors.
    //
    // ===========================================================

    /**
     * The default constructor.
     *
     * @param op - The operation to map.
     */

    public PSDLFactoryMapping(StorageHomeFactoryDecl factory)
    {
        // Init internal state
        factory_ = factory;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Map a factory definition into Java methods.
     *
     * @param generator - The PSDL to Java generator.
     * @param obj - The StorageHome mapping object.
     * @param ast - The abstract storage type defining this factory.
     * @param st - The storage type managed by the storage home.
     */
    public void
    toJava(org.objectweb.corba.generator.psdl.api.PSDL2JavaGenerator generator,
           InterfaceObject obj,
           AbstractStorageTypeDecl ast,
           StorageTypeDecl st)
    throws GenerationException
    {
        MethodObject method = null;
        String[] states = null;

        method = new MethodObjectImpl();
        method.addComment("The " + factory_.getName() + " factory.");
        method.setName(factory_.getName());
        method.setReturnType( generator.getTranslator().getAbsoluteName(ast) );
        states = factory_.getMemberList().getStrings();
        StateMemberMapping.mapStateListToParameters
            ( generator,
              states,
              method,
              ast );
        obj.addMethod(method);

        if (st != null)
        {
            StorageTypeStateMemberDecl state = null;
            java.util.List states_list = new java.util.ArrayList();
            for (int i=0; i<states.length; i++)
            {
                // Find the state in the current storageType
                state = (StorageTypeStateMemberDecl)generator.getDeclaration(ast, states[i]);

                if (state == null)
                {
                    AbstractStorageTypeDecl[] implemented = null;

                    // Find in the implemented classes
                    implemented = ast.getAllImplementedAbstractStorageTypes().getAbstractStorageTypes();
                    if(implemented.length > 0)
                    {
                        int j = 0;
                        while( (j<implemented.length) && (state == null) )
                        {
                            state = (StorageTypeStateMemberDecl)generator.getDeclaration(implemented[j], states[i]);
                            j++;
                        }
                    }
                    if (state == null)
                    {
                        String msg = "State Member '"+states[i]+"' not found!";
                        throw new GenerationException(msg);
                    }
                }
                states_list.add(state);
            }
            method.getImpl().setMacro("SH_CREATE1_METHOD");
            method.getImpl().addContextValue("states", states_list);
            method.getImpl().addContextValue("st_type", generator.getTranslator().getAbsoluteName(st));
        }
    }
}
