/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ________________________________________.

====================================================================*/

package org.objectweb.corba.generator.psdl.lib;

/** To access AST. */
import org.objectweb.corba.ast.api.PsdlOperationDecl;
import org.objectweb.corba.ast.api.Parameter;
import org.objectweb.corba.ast.api.ExceptionDecl;

/** To access Java AST. */
import org.objectweb.corba.generator.java.ast.api.*;
import org.objectweb.corba.generator.java.ast.lib.*;

/** Others. */
import org.objectweb.corba.generator.java.api.JavaTranslator;


/**
 * This class generates the mapping for a PSDL operation.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</A>
 */

public class PSDLOperationMapping
  implements org.objectweb.corba.generator.psdl.api.PSDLOperationMapping
{

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The PSDL operation Declaration. */
    private PsdlOperationDecl op_;

    // ===========================================================
    //
    // Constructors.
    //
    // ===========================================================

    /**
     * The default constructor.
     *
     * @param op - The operation to map.
     */

    public PSDLOperationMapping(PsdlOperationDecl op)
    {
        // Init internal state
        op_ = op;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Map a Psdl Operation.
     *
     * @param translator - A Java utility class to translate types.
     * @param obj - The interface (or class) to add the mapping in.
     */
    public void
    toJava(JavaTranslator translator,
           InterfaceObject obj)
    {
        MethodObject method = null;
        ParameterObject param = null;
        Parameter[] parameters = null;
        ExceptionDecl[] exceptions = null;

        method = new MethodObjectImpl();
        method.addComment("The "+op_.getName()+" method.");
        method.setName(op_.getName());
        method.setReturnType( translator.toJava(op_.getType()) );

        // Compute parameters
        parameters = op_.getParameterList().getParameters();
        for (int i=0; i<parameters.length; i++)
        {
            param = new ParameterObjectImpl();
            param.setName(parameters[i].getName());
            param.setType( translator.toJava(parameters[i].getType(),
                                             parameters[i].getMode()) );
            method.addParameter(param);
        }

        // Compute Exceptions
        exceptions = op_.getExceptionList().getExceptions();
        for (int i=0; i<exceptions.length; i++)
        {
            method.addException( translator.getAbsoluteName(exceptions[i]) );
        }

        obj.addMethod(method);
    }
}
