/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.openccm.browser.Components;

/** The console's imports. */
import org.objectweb.openccm.browser.CORBA.TypageCORBA;
import org.objectweb.util.browser.naming.api.Context;
import org.objectweb.util.browser.naming.wrappers.common.AbstractContext;
import org.objectweb.util.browser.naming.api.Name;
import org.objectweb.util.browser.naming.api.Entry;
import org.objectweb.util.browser.naming.api.EntryIterator;
import org.objectweb.util.browser.naming.wrappers.common.DefaultName;
import org.objectweb.util.browser.naming.wrappers.common.DefaultEntry;
import org.objectweb.util.browser.naming.wrappers.common.DefaultIterator;

/** The CCM's imports */
import org.omg.Components.CCMHome;
import org.omg.Components.CCMObject;

/**
 * Basic implementation of the Context interface
 * for managing CCMHome object.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */

public class HomeContext extends AbstractContext {

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The CCMHome. */
    private CCMHome home_ = null;

    // ==================================================================
    //
    // No Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    protected String getName(CCMObject o) {
        TypageCORBA tc = new TypageCORBA(o, org.objectweb.ccm.CORBA.TheORB.getORB());
        return tc.getTypeID();
    }

    // ==================================================================
    //
    // Public methods for interface Context.
    //
    // ==================================================================

    /**
     * Returns a new {@link EntryIterator EntryIterator} to
     * iterate on the entries contained by the target context.
     *
     * @return A new EntryIterator.
     */
    public EntryIterator getOWEntries() {
        CCMObject[] objects = home_.get_components();
        Entry[] values = new Entry[objects.length];
        for (int i = 0; i < objects.length; i++)
            values[i] = new DefaultEntry(objects[i], new DefaultName(getName(objects[i]) + " (" + i + ")"), this);
        return new DefaultIterator(values);
    }

    // ==================================================================
    //
    // Additional public methods for Wrapper interface.
    //
    // ==================================================================

    /**
     * Fix the wrapped object
     *
     * @param object The object to wrap
     *
     */
    public void setWrapped(Object object) {
        // TODO : narrow
        home_ = (CCMHome) object;
    }

    /**
     * Gets the wrapped object
     *
     * @return The wrapped object
     *
     */
    public Object getWrapped() {
        return home_;
    }

    /*
     * Gets the hashCode of the wrapped object
     *
     * @return The hashCode of the wrapped object
     * 
     */
    public int getWrappedHashCode() {
        if (home_ != null)
            return home_.hashCode();
        else
            return 0;
    }

}
