/*===========================================================================
ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jerome Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.openccm.browser.CosNaming;

/** The administration console's API imports */
import org.objectweb.util.browser.naming.api.Context;
import org.objectweb.util.browser.naming.wrappers.common.AbstractContext;
import org.objectweb.util.browser.naming.api.Name;
import org.objectweb.util.browser.naming.api.Entry;
import org.objectweb.util.browser.naming.api.EntryIterator;
import org.omg.CosNaming.NamingContext;

/** The Java API's imports */
import java.util.StringTokenizer;

/**
 * Basic implementation of the Context interface
 * for managing NamingContext
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class NameServiceContext extends AbstractContext {

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The Naming context. */
    private NamingContext object_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Obtains the associated naming context.
     *
     * @return The associated naming context.
     */
    protected NamingContext getNamingContext() {
        return object_;
    }

    /** 
     * This object represents an entry in the name service
     * as "id:kind"
     */
    protected class NameServiceEntry {

        /** The id of the entry */
        protected String id_;

        /** The kind of the entry */
        protected String kind_;

        /** Constructs a NameServiceEntry
         * @param entry The entry constructs as "id:kind"
         */
        public NameServiceEntry(String entry) {
            StringTokenizer str = new StringTokenizer(entry, ":");
            if (str.hasMoreTokens()) {
                id_ = str.nextToken();
                if (str.hasMoreTokens()) {
                    kind_ = str.nextToken();
                }
            }
        }

        /**
         * Returns the id of the entry
         */
        public String getId() {
            return id_;
        }

        /**
         * Returns the kind of the entry
         */
        public String getKind() {
            return (kind_ == null) ? "" : kind_;
        }

    }

    // ==================================================================
    //
    // Public methods for interface Context.
    //
    // ==================================================================

    /**
     * Gets the scope of the target context.
     *
     * This method does not update the scope of the target context.
     *
     * @return The scope of the target context, or null if the
     *         target context has no scope.
     *
     * @postcondition return == getOWScope();
     */
    public Context getOWScope() {
        // TODO
        return null;
    }

    /**
     * Gets a local entry in the target context identified by <code>name</code>,
     * or null if no entry exists under name <code>name</code>.
     *
     * Here is a local resolution in the target context.
     *
     * This method does not modify the target context.
     *
     * @param name The name of the entry to be retrieved.
     *
     * @return The corresponding entry, if any; or null otherwise.
     *
     * @postcondition
     *      result.getOWName().getOWId().equals(name)
     *  and result.getOWContext() == this
     *  and this.isNotModified();
     */
    public Entry getOWLocalEntry(String name) {
        if (getNamingContext() != null) {
            // ???
            Object value = getOWLocalValue(name);
            NameServiceEntry entry = new NameServiceEntry(name);
            // Returns a created entry if name is found, otherwise null.
            return (value != null) ? createEntry(value, entry.getId()) : null;
        }
        return null;
    }

    /**
     * Gets the value of a local entry in the target context identified by
     * <code>name</code>, or null if no entry exists under name
     * <code>name</code>.
     *
     * This method is equivalent to
     * {@link #getOWLocalEntry(String) <code>getOWLocalEntry(name)</code>}
     * {@link Entry#getOWValue() <code>.getOWValue()</code>}.
     *
     * This method does not modify the target context.
     *
     * @param name The name of the entry to be retrieved. 
     * The form of the informations are "id:kind"
     *
     * @return The corresponding entry value, if any; or null otherwise.
     *
     * @postcondition
     *     ( getOWLocalEntry(name) == null && result == null )
     *  or result == getOWLocalEntry(name).getOWValue());
     */
    public Object getOWLocalValue(String name) {
        if (getNamingContext() != null) {
            org.omg.CORBA.Object obj = null;
            org.omg.CosNaming.NameComponent[] nameComponent = new org.omg.CosNaming.NameComponent[1];
            nameComponent[0] = new org.omg.CosNaming.NameComponent();

            NameServiceEntry entry = new NameServiceEntry(name);

            nameComponent[0].id = entry.getId();
            nameComponent[0].kind = entry.getKind();
            try {
                obj = object_.resolve(nameComponent);
            } catch (org.omg.CosNaming.NamingContextPackage.NotFound e) {
                System.out.println("Objet non trouve");
            } catch (org.omg.CosNaming.NamingContextPackage.CannotProceed e) {
                System.out.println("Impossible d'effectuer le traitement");
            } catch (org.omg.CosNaming.NamingContextPackage.InvalidName e) {
                System.out.println("Nom Invalide");
            }
            return obj;
        }
        return null;
    }

    /**
     * Gets the entry in the target context identified by <code>name</code>,
     * or null if no entry exists under name <code>name</code>.
     *
     * This method does not modify the given name and the target context.
     *
     * @param name The name of the entry to be retrieved.
     *
     * @return The corresponding entry, if any; or null otherwise.
     *
     * @postcondition
     *      result.getOWName().equals(name)
     *  and name.isNotModified()
     *  and this.isNotModified();
     */
    public Entry getOWEntry(Name name) {
        // TODO
        return null;
    }

    /**
     * Gets the value of an entry in the target context identified by
     * <code>name</code>, or null if no entry exists under name
     * <code>name</code>.
     *
     * This method is equivalent to
     * {@link #getOWEntry(Name) <code>getOWEntry(name)</code>}
     * {@link Entry#getOWValue() <code>.getOWValue()</code>}.
     *
     * This method does not modify the given name and the target context.
     *
     * @param name The name of the entry to be retrieved.
     *
     * @return The corresponding entry, if any; or null otherwise.
     *
     * @postcondition
     *     ( getOWEntry(name) == null && result == null )
     *  or result == getOWEntry(name).getOWValue());
     */
    public Object getOWValue(Name name) {
        // TODO
        return null;
    }

    /**
     * Returns a new {@link EntryIterator EntryIterator} to
     * iterate on the entries contained by the target context.
     *
     * @return A new EntryIterator.
     */
    public EntryIterator getOWEntries() {
        return new NameServiceIterator(this);
    }

    // ==================================================================
    //
    // Additional public methods for Wrapper interface.
    //
    // ==================================================================

    /**
     * Fix the wrapped object
     *
     * @param object The object to wrap
     *
     */
    public void setWrapped(Object object) {
        object_ = (NamingContext) object;
    }

    /**
     * Gets the wrapped object
     *
     * @return The wrapped object
     *
     */
    public Object getWrapped() {
        return object_;
    }

    /**
     * Gets the hashCode of the wrapped object
     *
     * @return The hashCode of the wrapped object
     * 
     */
    public int getWrappedHashCode() {
        if (object_ != null)
            return object_.hashCode();
        else
            return 0;
    }
}
