/*===========================================================================

ObjectWeb Browser Framework
Copyright (C) 2003 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jrme Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.openccm.browser.Deployment;

import java.awt.Component;
import javax.swing.JOptionPane;
import java.io.File;
import java.io.BufferedReader;
import java.io.FileReader;
import javax.swing.Box;

import org.objectweb.openccm.browser.menu.LabelBox;
import org.objectweb.openccm.browser.menu.FileChooserBox;
import org.objectweb.openccm.browser.menu.OpenCCMBrowserConstants;
import org.objectweb.util.browser.api.ExtendedActionEvent;
import org.objectweb.util.browser.api.ExtendedActionListener;
import org.omg.Components.Deployment.Container;

/**
 * Installs a new home from a Container.
 * You have to specify : 
 * <ul>
 * <li>The UUID id.</li>
 * <li>The entry point.</li>
 * <li>The config file of the home.</li>
 * </ul>
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class InstallHome implements ExtendedActionListener {

    /** The box to specify id */
    protected LabelBox id_;

    /** The box to specify entry point */
    protected LabelBox entryPoint_;

    /** The box to specify the config file */
//    protected FileChooserBox config_;

    /**
     * Returns a String which contains all the file contents.
     */
    protected String fileToString(File f) throws Exception {
        StringBuffer bf = new StringBuffer();
        BufferedReader in = new BufferedReader(new FileReader(f));
        String line = "";
        while (line != null) {
            bf.append(line);
            line = in.readLine();
        }
        return bf.toString();
    }

    /**
     * Create a box containing all the box to specify all the params
     */
    protected Box createBox() {
        Box box = Box.createVerticalBox();
        box.add(Box.createVerticalGlue());
        id_ = new LabelBox("UUID : ");
        box.add(id_);
        box.add(Box.createVerticalStrut(10));
        entryPoint_ = new LabelBox("Entry Point : ");
        box.add(entryPoint_);
//        box.add(Box.createVerticalStrut(10));
//        config_ = new FileChooserBox("Config File : ", 0);
//        box.add(config_);
        box.add(Box.createVerticalGlue());
        return box;
    }

    public void actionPerformed(ExtendedActionEvent e) throws Exception {
        Container container = (Container) e.getCurrentValue();
        Object[] options = { "Install", "Cancel" };
        int result =
            JOptionPane.showOptionDialog((Component) e.getActionEvent().getSource(), createBox(), "Install a home", JOptionPane.DEFAULT_OPTION, JOptionPane.PLAIN_MESSAGE, null, options, options[0]);
        if (result == 0) {
            String UUID = id_.getLabel();
            String entryPt = entryPoint_.getLabel();
//            File f = config_.getFile();
//            if (UUID != null && !UUID.equals("") && entryPt != null && !entryPt.equals("") && f != null && f.exists()) {
            if (UUID != null && !UUID.equals("") && entryPt != null && !entryPt.equals("")) {
//                org.omg.Components.ConfigValue[] config = new org.omg.Components.ConfigValue[1];
                org.omg.Components.ConfigValue[] config = new org.omg.Components.ConfigValue[0];
//                config[0] = new org.objectweb.ccm.Components.ConfigValueImpl();
//                config[0].name = "home_script";
//                org.omg.CORBA.TypeCode string_tc = org.objectweb.ccm.CORBA.TheORB.getORB().get_primitive_tc(org.omg.CORBA.TCKind.tk_string);
//                org.omg.CORBA.Any value = org.objectweb.ccm.CORBA.TheDynamicAnyFactory.getFactory().create_dyn_any_from_type_code(string_tc).to_any();
//                value.insert_string(fileToString(f));
//                config[0].value = value;
                // Installs the home
                container.install_home(UUID, entryPt, config);
            } else {
                throw new Exception("At least one parameter is missing !");
            }
        }
    }

}




