/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/
package org.objectweb.openccm.browser;

/** The administration console's API*/
import org.objectweb.util.browser.naming.wrappers.common.DefaultContextContainer;
import org.objectweb.util.browser.naming.api.ContextContainer;
import org.objectweb.util.browser.api.TreeConfiguration;
import org.objectweb.util.browser.gui.core.DefaultTreePanel;
import org.objectweb.util.browser.gui.core.DefaultViewPanel;
import org.objectweb.util.browser.gui.core.DynamicTree;

import org.objectweb.openccm.browser.menu.Menu;
import org.objectweb.openccm.browser.menu.TreeDialogSingleton;

/** The java API's imports */
import java.io.File;
import java.awt.Dimension;
import java.awt.BorderLayout;
import javax.swing.JFrame;
import javax.swing.JSplitPane;

/**
 * This class defines the main frame
 * 
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class Main extends JFrame {

    /**
     * Populates the tree
     */
    protected void populateTree(DynamicTree tree) {

        // Adds the name service into the initial context
        try {
            org.omg.CORBA.Object o = org.objectweb.ccm.CORBA.TheORB.getORB().resolve_initial_references("NameService");
            tree.addToInitialContext("NameService", o, 2);
        } catch (org.omg.CORBA.ORBPackage.InvalidName e1) {
            // NS is not started
        } catch (org.omg.CORBA.TRANSIENT e1) {
	    // Ignore (throws when the NameService is not started)
	}

        // Adds the interface repository into the initial context
        try {
            org.omg.CORBA.Object o = org.objectweb.ccm.CORBA.TheORB.getORB().resolve_initial_references("InterfaceRepository");
            tree.addToInitialContext("InterfaceRepository", o);
        } catch (org.omg.CORBA.ORBPackage.InvalidName e1) {
            // IR is not started
        }

    }

    protected void initializeTreeDialog(TreeConfiguration treeConfiguration) {
        DynamicTree tree = new DynamicTree();
        tree.setNewBrowserProperty(treeConfiguration.getBrowserProperty());
        tree.setNewContextProperty(treeConfiguration.getContextProperty());

        // Adds the name service into the initial context
        try {
            org.omg.CORBA.Object o = org.objectweb.ccm.CORBA.TheORB.getORB().resolve_initial_references("NameService");
            tree.addToInitialContext("NameService", o, 2);
        } catch (org.omg.CORBA.ORBPackage.InvalidName e1) {
            // NS is not strated
        } catch (org.omg.CORBA.TRANSIENT e1) {
	    // Ignore (throws when the NameService is not started)
	}

        TreeDialogSingleton.setInstance(tree);
    }

    /**
     * Default constructor
     */
    public Main(File contextPropertyFile, File[] browserPropertyFile) {

        super("Admin GUI");

        DefaultViewPanel viewPanel = new DefaultViewPanel();

        DynamicTree tree = new DynamicTree();
        tree.setNewContextProperty(contextPropertyFile);
        tree.setNewBrowserProperty(browserPropertyFile);
        populateTree(tree);
        tree.setViewPanel(viewPanel);

        // Initializes the tree using in JDialog Boxes,
        initializeTreeDialog((TreeConfiguration) tree);

        // Create the menu
        Menu menu = new Menu(tree);
        setJMenuBar(menu.getMenuBar());

        getContentPane().setLayout(new BorderLayout());
        //		getContentPane().add(menu.getToolBar(),BorderLayout.NORTH);
        getContentPane().add(new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, true, new DefaultTreePanel(tree), viewPanel), BorderLayout.CENTER);

        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        pack();

        // Centers the frame
        Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setLocation((screenSize.width - getWidth()) / 2, (screenSize.height - getHeight()) / 2);

        setVisible(true);

    }

}
