/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jerome Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.openccm.browser.menu;

/** The java API's imports */
import javax.swing.JPanel;
import javax.swing.filechooser.FileFilter;
import javax.swing.JFileChooser;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.AbstractAction;
import javax.swing.Box;
import javax.swing.SwingConstants;

import org.objectweb.util.browser.gui.core.DynamicTree;
import org.objectweb.util.browser.naming.api.Entry;
import org.objectweb.util.browser.naming.api.ExtendedEntry;

import org.objectweb.openccm.browser.menu.TreeDialogSingleton;

import java.awt.Component;
import java.awt.Dimension;
import java.io.File;

/**
 * This class represents the panel while allow to choose a file and specify a label to the entry
 * 
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class BrowserPanel extends JPanel {

    /** The associated FileChooserPanel */
    protected FileChooserPanel fileChooserPanel_;

    /** The associated Tree */
    protected DynamicTree tree_;

    /** 
     * Default constructor
     */
    public BrowserPanel(String nameFieldLabel, String nameBrowserLabel, int fileFilter) {

        fileChooserPanel_ = new FileChooserPanel(nameFieldLabel, nameBrowserLabel, fileFilter);

        Box b = Box.createVerticalBox();
        b.add(Box.createVerticalGlue());

        b.add(fileChooserPanel_);

        b.add(Box.createVerticalStrut(10));

        tree_ = TreeDialogSingleton.getInstance();
        Box treeBox = Box.createHorizontalBox();
        //tree.setPreferredSize(new Dimension(350,150));
        JScrollPane scroll = new JScrollPane(tree_);
        scroll.setPreferredSize(new Dimension(350, 150));
        treeBox.add(scroll);
        b.add(treeBox);

        b.add(Box.createVerticalGlue());

        add(b);
    }

    /** Returns the specify label */
    public String getLabel() {
        return fileChooserPanel_.getLabel();
    }

    /** Returns the chosen file */
    public File getFile() {
        return fileChooserPanel_.getFile();
    }

    /** Returns the selected object */
    public Object getSelectedObject() {
        if (tree_ != null) {
            Entry entry = tree_.getSelectedEntry();
            if (entry != null)
                return (((ExtendedEntry) entry).getWrappedObject());
        }
        return null;
    }

}
