/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jerome Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.openccm.browser.menu;

/** The java API's imports */
import javax.swing.filechooser.FileFilter;
import javax.swing.JFileChooser;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.AbstractAction;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.SwingConstants;

import java.awt.Component;
import java.awt.Dimension;
import java.io.File;

/**
 * This class represents the panel while allow to choose a file.
 * 
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class FileChooserBox extends Box {

    /** The file chooser that allow to find the file to load */
    protected JFileChooser fileChooser_;

    /** The chosen file */
    protected File file_;

    /** 
     * Default constructor
     */
    public FileChooserBox(String label, int fileFilterType) {
        super(BoxLayout.X_AXIS);
        fileChooser_ = JFileChooserSingleton.getInstance(fileFilterType);
        add(Box.createHorizontalGlue());
        JLabel browserLabel = new JLabel(label, SwingConstants.RIGHT);
        browserLabel.setAlignmentX(Component.RIGHT_ALIGNMENT);
        browserLabel.setAlignmentY(Component.CENTER_ALIGNMENT);
        add(browserLabel);
        add(Box.createHorizontalStrut(5));
        JTextField fileName = new JTextField();
        fileName.setEditable(false);
        fileName.setPreferredSize(new Dimension(200, 20));
        fileName.setMaximumSize(new Dimension(200, 20));
        add(fileName);
        add(Box.createHorizontalStrut(5));
        JButton browseButton = new JButton(new BrowseAction(this, fileName));
        browseButton.setPreferredSize(new Dimension(20, 20));
        browseButton.setMaximumSize(new Dimension(20, 20));
        add(browseButton);
    }

    /** Returns the chosen file */
    public File getFile() {
        return file_;
    }

    /**
     * Action which 
     */
    protected class BrowseAction extends AbstractAction {

        /** The parent of the panel */
        protected Component parent_;

        /** */
        protected JTextField textField_;

        /**
         * Default constructor
         */
        protected BrowseAction(Component parent, JTextField textField) {
            super("...", null);
            parent_ = parent;
            textField_ = textField;
        }

        public void actionPerformed(java.awt.event.ActionEvent ae) {
            int returnVal = fileChooser_.showOpenDialog(parent_);
            if (returnVal == JFileChooser.APPROVE_OPTION) {
                file_ = fileChooser_.getSelectedFile();
                textField_.setText(fileChooser_.getSelectedFile().toString());
            }
        }

    }

}
