/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jerome Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.openccm.browser.menu;

/** The administration console's imports */
import org.objectweb.util.browser.gui.core.DynamicTree;

/** The java API's imports */
import javax.swing.JOptionPane;
import java.io.File;
import java.awt.Component;
import javax.swing.ImageIcon;
import java.io.BufferedReader;
import java.io.FileReader;

/** The CORBA's imports */
import org.omg.CORBA.ORB;

/**
 * This class represents the action which load a new configuration file
 * 
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class LoadIORFileAction extends AbstractMenuAction {

    /**
     * The default constructor
     */
    public LoadIORFileAction(String nom, ImageIcon image, String desc, Integer mnemonic, DynamicTree tree, Component parent) {
        super(nom, image, desc, mnemonic, tree);
    }

    /**
     * Shows a JFileChooser component and tries to load the new configuration file
     */
    public void actionPerformed(java.awt.event.ActionEvent ae) {

        BrowserPanel bp = new BrowserPanel("Object's name : ", "IOR file : ", OpenCCMBrowserConstants.IOR_FILE_CHOOSER);

        int result = JOptionPane.showOptionDialog((Component) ae.getSource(), bp, "Adds an IOR class", JOptionPane.OK_CANCEL_OPTION, JOptionPane.PLAIN_MESSAGE, null, null, null);

        if (result == 0) {
            File file = bp.getFile();
            Object selectedObject = bp.getSelectedObject();
            org.omg.CORBA.Object object = null;
            if (file != null) {
                String ior = null;
                try {
                    BufferedReader in = new BufferedReader(new FileReader(file));
                    ior = in.readLine();
                } catch (java.io.FileNotFoundException err) {
                    System.out.println(file + " : File not found !");
                } catch (java.io.IOException err) {
                    System.out.println(file + " : Error while reading !");
                }
                if (ior != null && ior.startsWith("IOR:")) {
                    ORB orb = org.objectweb.ccm.CORBA.TheORB.getORB();
                    object = orb.string_to_object(ior);
                }
            } else if (selectedObject != null) {
                try {
                    object = (org.omg.CORBA.Object) selectedObject;
                } catch (ClassCastException e1) {
                    JOptionPane.showMessageDialog(null, "You must select a org.omg.CORBA.Object \n Found : " + selectedObject.getClass().getName(), "Bad Object !", JOptionPane.ERROR_MESSAGE);
                }
            }
            if (object != null && bp.getLabel() != null && bp.getLabel().length() > 0)
                tree_.addToInitialContext(bp.getLabel(), object);
            else
                JOptionPane.showMessageDialog(null, "One parameter is missing !", "Error !", JOptionPane.ERROR_MESSAGE);
        }
    }
}
