/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jerome Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.gui.panel;

/** The console's imports */
import org.objectweb.util.browser.naming.api.Context;
import org.objectweb.util.browser.naming.api.Entry;
import org.objectweb.util.browser.naming.api.EntryIterator;
import org.objectweb.util.browser.gui.panel.NodeViewPanel;
import org.objectweb.util.browser.gui.common.TreeView;

/** The Java API's imports */
import java.awt.Color;
import javax.swing.Box;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;

/**
 * Panel which display a context
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */

public class ContextPanel extends NodeViewPanel {
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    protected Box box_ = null;

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================    

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // No internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // No public methods.
    //
    // ==================================================================

    /**
     * Empty constructor
     */
    public ContextPanel() {
        setBackground(Color.white);
        box_ = Box.createVerticalBox();
        add(box_);
    }

    /**
     * Invokes just after instanciation
     */
    public void preInitialize() {
    }

    /**
     * Fix the view which allow to manipulate the tree
     *
     * @param treeView The view on the tree
     */
    public void setTreeView(TreeView treeView) {
        Context context = (Context) treeView.getHandleObject();
        if (context != null) {
            EntryIterator entries = context.getOWEntries();
            while (entries.hasNext()) {
                Entry entry = entries.next();
                String text = entry.getOWValue().toString();
                JTextField textField = new JTextField(text);
                textField.setEditable(false);
                textField.setBackground(Color.white);
                textField.setBorder(new TitledBorder(null,entry.getOWName().getOWId(),TitledBorder.LEFT,TitledBorder.TOP));
                box_.add(textField);
            }
        }
    }

    /**
     * Invokes just before removing
     */
    public void postInitialize() {
    }

}
