/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Offroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.monolog.provider.lib;

import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;
import org.objectweb.util.monolog.provider.api.LoggerProvider;

/**
 * @author Jerome Offroy
 * Singleton class for LoggerProviderImpl
 */
public class LoggerProviderSingleton {

    /**
     * The Singleton LoggerProviderImpl
     */
    static private LoggerProvider unique = null;

    /**
     * Get the unique instance.
     * Create it at first call.
     */
    static public LoggerProvider getLoggerProvider() {
        if (unique == null) {
            unique = new LoggerProviderImpl();
        }
        return unique;
    }

    /**
     * Method getLogger.
     * @param topic
     * @param caller
     * @return Logger
     */
    static public Logger getLogger(String topic, Class caller) {
        return getLoggerProvider().getLogger(topic, caller);
    }

    /**
     * Method getLoggerFactory.
     * @return LoggerFactory
     */
    static public LoggerFactory getLoggerFactory() {
        return unique.getLoggerFactory();
    }

    /**
     * Method getLogger.
     * @param topic
     * @return Logger
     */
    static public Logger getLogger(String topic) {
        return unique.getLogger(topic);
    }

    static public Logger createLogger(String topic, Class caller, String []description){        
        return getLoggerProvider().createLogger(topic,caller,description);
    }

}
